@@ -14,24 +14,6 @@ module Sequel
     end
 
     module Postgres
-      # Return PostgreSQL array types as ruby Arrays instead of
-      # JDBC PostgreSQL driver-specific array type. Only used if the
-      # database does not have a conversion proc for the type.
-      def self.RubyPGArray(r, i)
-        if v = r.getArray(i)
-          v.array.to_ary
-        end
-      end 
-
-      # Return PostgreSQL hstore types as ruby Hashes instead of
-      # Java HashMaps.  Only used if the database does not have a
-      # conversion proc for the type.
-      def self.RubyPGHstore(r, i)
-        if v = r.getObject(i)
-          v.to_hash
-        end
-      end 
-
       module DatabaseMethods
         include Sequel::Postgres::DatabaseMethods
 
@@ -135,11 +117,6 @@ module Sequel
 
         private
         
-        DATABASE_ERROR_CLASSES = [NativeException].freeze
-        def database_error_classes
-          DATABASE_ERROR_CLASSES
-        end
-
         def disconnect_error?(exception, opts)
           super || exception.message =~ /\A(This connection has been closed\.|FATAL: terminating connection due to administrator command|An I\/O error occurred while sending to the backend\.)\z/
         end
@@ -187,10 +164,10 @@ module Sequel
         end
 
         # Execute the connection configuration SQL queries on the connection.
-        def setup_connection(conn)
-          conn = super(conn)
+        def setup_connection_with_opts(conn, opts)
+          conn = super
           statement(conn) do |stmt|
-            connection_configuration_sqls.each{|sql| log_connection_yield(sql, conn){stmt.execute(sql)}}
+            connection_configuration_sqls(opts).each{|sql| log_connection_yield(sql, conn){stmt.execute(sql)}}
           end
           conn
         end
@@ -203,6 +180,12 @@ module Sequel
       
       class Dataset < JDBC::Dataset
         include Sequel::Postgres::DatasetMethods
+
+        # Warn when calling as the fetch size is ignored by the JDBC adapter currently.
+        def with_fetch_size(size)
+          warn("Sequel::JDBC::Postgres::Dataset#with_fetch_size does not currently have an effect.", :uplevel=>1)
+          super
+        end
         
         private
         
@@ -218,9 +201,17 @@ module Sequel
 
         STRING_TYPE = Java::JavaSQL::Types::VARCHAR
         ARRAY_TYPE = Java::JavaSQL::Types::ARRAY
-        ARRAY_METHOD = Postgres.method(:RubyPGArray)
-        PG_SPECIFIC_TYPES = [ARRAY_TYPE, Java::JavaSQL::Types::OTHER, Java::JavaSQL::Types::STRUCT].freeze
-        HSTORE_METHOD = Postgres.method(:RubyPGHstore)
+        PG_SPECIFIC_TYPES = [Java::JavaSQL::Types::ARRAY, Java::JavaSQL::Types::OTHER, Java::JavaSQL::Types::STRUCT, Java::JavaSQL::Types::TIME_WITH_TIMEZONE, Java::JavaSQL::Types::TIME].freeze
+
+        # Return PostgreSQL hstore types as ruby Hashes instead of
+        # Java HashMaps.  Only used if the database does not have a
+        # conversion proc for the type.
+        HSTORE_METHOD = Object.new
+        def HSTORE_METHOD.call(r, i)
+          if v = r.getObject(i)
+            v.to_hash
+          end
+        end 
 
         def type_convertor(map, meta, type, i)
           case type
@@ -228,8 +219,6 @@ module Sequel
             oid = meta.getField(i).getOID
             if pr = db.oid_convertor_proc(oid)
               pr
-            elsif type == ARRAY_TYPE
-              ARRAY_METHOD
             elsif oid == 2950 # UUID
               map[STRING_TYPE]
             elsif meta.getPGType(i) == 'hstore'
