@@ -36,7 +36,6 @@ module Sequel
       # options such as :local_infile.
       def connect(server)
         opts = server_opts(server)
-        opts[:host] ||= 'localhost'
         opts[:username] ||= opts.delete(:user)
         opts[:flags] ||= 0
         opts[:flags] |= ::Mysql2::Client::FOUND_ROWS if ::Mysql2::Client.const_defined?(:FOUND_ROWS)
@@ -78,8 +77,8 @@ module Sequel
       end
 
       # Return the version of the MySQL server to which we are connecting.
-      def server_version(server=nil)
-        @server_version ||= (synchronize(server){|conn| conn.server_info[:id]} || super)
+      def server_version(_server=nil)
+        @server_version ||= super()
       end
 
       private
@@ -195,7 +194,8 @@ module Sequel
       end
 
       def database_exception_sqlstate(exception, opts)
-        exception.sql_state
+        state = exception.sql_state
+        state unless state == 'HY000'
       end
 
       def dataset_class_default
@@ -244,6 +244,9 @@ module Sequel
       # it hasn't been disabled.
       def paged_each(opts=OPTS, &block)
         if STREAMING_SUPPORTED && opts[:stream] != false
+          unless block_given?
+            return enum_for(:paged_each, opts)
+          end
           stream.each(&block)
         else
           super
