@@ -38,7 +38,6 @@ module Sequel
         @constraints = []
         @primary_key = nil
         instance_exec(&block) if block
-        @columns.unshift(@primary_key) if @primary_key && !has_column?(primary_key_name)
       end
 
       # Use custom Bignum method to use :Bignum instead of Bignum class, to work
@@ -67,7 +66,7 @@ module Sequel
             method = type.to_s
           end
 
-          define_method(method){|name, opts={}| column(name, type, opts)}
+          define_method(method){|name, opts=OPTS| column(name, type, opts)}
         end
         nil
       end
@@ -81,7 +80,8 @@ module Sequel
         constraint(nil, *args, &block)
       end
 
-      # Add a column with the given name, type, and opts      #
+      # Add a column with the given name, type, and opts:
+      #
       #   column :num, :integer
       #   # num INTEGER
       #
@@ -109,6 +109,9 @@ module Sequel
       #                yet exist on referenced table (but will exist before the transaction commits).
       #                Basically it adds DEFERRABLE INITIALLY DEFERRED on key creation.
       #                If you use :immediate as the value, uses DEFERRABLE INITIALLY IMMEDIATE.
+      # :generated_always_as :: Specify a GENERATED ALWAYS AS column expression,
+      #                         if generated columns are supported (PostgreSQL 12+, MariaDB 5.2.0+,
+      #                         and MySQL 5.7.6+).
       # :index :: Create an index on this column.  If given a hash, use the hash as the
       #           options for the index.
       # :key :: For foreign key columns, the column in the associated table
@@ -122,18 +125,36 @@ module Sequel
       # :on_update :: Specify the behavior of this column when being updated
       #               (:restrict, :cascade, :set_null, :set_default, :no_action).
       # :primary_key :: Make the column as a single primary key column.  This should not
-      #                 be used if you have a single, nonautoincrementing primary key column
+      #                 be used if you want a single autoincrementing primary key column
       #                 (use the primary_key method in that case).
       # :primary_key_constraint_name :: The name to give the primary key constraint
+      # :primary_key_deferrable :: Similar to :deferrable, but for the primary key constraint
+      #                            if :primary_key is used.
       # :type :: Overrides the type given as the argument.  Generally not used by column
       #          itself, but can be passed as an option to other methods that call column.
       # :unique :: Mark the column as unique, generally has the same effect as
       #            creating a unique index on the column.
       # :unique_constraint_name :: The name to give the unique key constraint
+      # :unique_deferrable :: Similar to :deferrable, but for the unique constraint if :unique
+      #                       is used.
+      #
+      # PostgreSQL specific options:
+      #
+      # :identity :: Create an identity column.
+      #
+      # MySQL specific options:
+      #
+      # :generated_type :: Set the type of column when using :generated_always_as,
+      #                    should be :virtual or :stored to force a type.
+      #
+      # Microsoft SQL Server specific options:
+      #
+      # :clustered :: When using :primary_key or :unique, marks the primary key or unique
+      #               constraint as CLUSTERED (if true), or NONCLUSTERED (if false).
       def column(name, type, opts = OPTS)
         columns << {:name => name, :type => type}.merge!(opts)
         if index_opts = opts[:index]
-          index(name, index_opts.is_a?(Hash) ? index_opts : {})
+          index(name, index_opts.is_a?(Hash) ? index_opts : OPTS)
         end
         nil
       end
@@ -152,7 +173,7 @@ module Sequel
         nil
       end
       
-      # Add a foreign key in the table that references another table. See column
+      # Add a foreign key in the table that references another table. See #column
       # for available options.
       #
       #   foreign_key(:artist_id) # artist_id INTEGER
@@ -216,6 +237,9 @@ module Sequel
       #                  operations on the table while the index is being
       #                  built.
       # :opclass :: Use a specific operator class in the index.
+      # :include :: Include additional column values in the index, without
+      #             actually indexing on those values (PostgreSQL 11+).
+      # :tablespace :: Specify tablespace for index.
       #
       # Microsoft SQL Server specific options:
       #
@@ -232,7 +256,7 @@ module Sequel
         nil
       end
       
-      # Add a column with the given type, name, and opts.  See +column+ for available
+      # Add a column with the given type, name, and opts.  See #column for available
       # options.
       def method_missing(type, name = nil, opts = OPTS)
         name ? column(name, type, opts) : super
@@ -263,8 +287,7 @@ module Sequel
       # 
       # Examples:
       #   primary_key(:id)
-      #   primary_key(:id, Bigint)
-      #   primary_key(:id, Bigint, keep_order: true)
+      #   primary_key(:id, type: :Bignum, keep_order: true)
       #   primary_key([:street_number, :house_number], name: :some constraint_name)
       def primary_key(name, *args)
         return composite_primary_key(name, *args) if name.is_a?(Array)
@@ -312,7 +335,7 @@ module Sequel
 
       # Add a composite primary key constraint
       def composite_primary_key(columns, *args)
-        opts = args.pop || {}
+        opts = args.pop || OPTS
         constraints << {:type => :primary_key, :columns => columns}.merge!(opts)
         nil
       end
@@ -348,7 +371,8 @@ module Sequel
       end
       
       # Add a column with the given name, type, and opts.
-      # See CreateTableGenerator#column for the available options.
+      # See CreateTableGenerator#column for the available options (except for +:index+, use a
+      # separate +add_index+ call to add an index for the column).
       #
       #   add_column(:name, String) # ADD COLUMN name varchar(255)
       #
@@ -390,7 +414,8 @@ module Sequel
       end
 
       # Add a foreign key with the given name and referencing the given table.
-      # See CreateTableGenerator#column for the available options.
+      # See CreateTableGenerator#column for the available options (except for +:index+, use a
+      # separate +add_index+ call to add an index for the column).
       #
       # You can also pass an array of column names for creating composite foreign
       # keys. In this case, it will assume the columns exist and will only add
@@ -623,7 +648,7 @@ module Sequel
 
       # Drop a composite foreign key constraint
       def drop_composite_foreign_key(columns, opts)
-        @operations << {:op => :drop_constraint, :type => :foreign_key, :columns => columns}.merge!(opts)
+        @operations << opts.merge(:op => :drop_constraint, :type => :foreign_key, :columns => columns)
         nil
       end
     end
