@@ -22,7 +22,7 @@ describe "A MSSQL database" do
     @db.dataset.server_version
   end
 end
-  
+
 describe "A MSSQL database" do
   before do
     @db = DB
@@ -40,6 +40,72 @@ describe "A MSSQL database" do
   end
 end
 
+describe "A MSSQL database" do
+  before do
+    @db = DB
+  end
+  after do
+    @db.drop_table?(:test3)
+  end
+
+  cspecify "should not modify column type when adding primary key", [:odbc] do
+    @db.create_table(:test3) do
+      column :row_id,   "int",              null: false, auto_increment: true
+      column :deviceid, "binary", size: 7,  null: false
+
+      index [:row_id], type: :clustered, unique: true
+    end
+    @db.alter_table(:test3) do
+      add_primary_key [:deviceid]
+    end
+    @db[:test3].insert(:deviceid=>Sequel.blob('abcdefg'))
+    @db[:test3].get(:deviceid).must_equal 'abcdefg'
+  end
+
+  it "should allow creating clustered and non-clustered primary keys" do
+    [true, false].each do |clustered|
+      @db.create_table!(:test3) do
+        primary_key :row_id, :clustered=>clustered
+        String :name
+      end
+    end
+  end
+
+  it "should allow creating clustered and non-clustered unique constraints" do
+    [true, false].each do |clustered|
+      @db.create_table!(:test3) do
+        Integer :row_id
+        String :name
+        unique [:name], :clustered=>clustered
+      end
+    end
+  end
+
+  it "should allow adding clustered and non-clustered primary keys" do
+    [true, false].each do |clustered|
+      @db.create_table!(:test3) do
+        Integer :row_id, :null=>false
+        String :name
+      end
+      @db.alter_table(:test3) do
+        add_primary_key [:row_id], :clustered=>clustered
+      end
+    end
+  end
+
+  it "should allow adding clustered and non-clustered unique constraints" do
+    [true, false].each do |clustered|
+      @db.create_table!(:test3) do
+        Integer :row_id, :null=>false
+        String :name
+      end
+      @db.alter_table(:test3) do
+        add_unique_constraint [:row_id], :clustered=>clustered
+      end
+    end
+  end
+end
+
 describe "MSSQL decimal locale handling" do
   before do
     @locale = WIN32OLE.locale
@@ -329,6 +395,18 @@ describe "Offset support" do
     @ds.limit(2, 2).all
     @hs.must_equal [{:id=>3, :parent_id=>1}, {:id=>4, :parent_id=>1}]
   end
+
+  it "should support returning limited results with ties" do
+    @ds.delete
+    @ds.insert(:id => 1)
+    @ds.insert(:id => 1)
+    @ds.insert(:id => 2)
+    @ds.insert(:id => 2)
+    @ds.order(:id).select(:id).limit(1).select_order_map(:id).must_equal [1]
+    @ds.order(:id).select(:id).limit(1).with_ties.select_order_map(:id).must_equal [1, 1]
+    @ds.order(:id).select(:id).limit(2, 1).select_order_map(:id).must_equal [1, 2]
+    proc{@ds.order(:id).select(:id).limit(2, 1).with_ties.select_order_map(:id)}.must_raise Sequel::Error
+  end
 end
 
 describe "Update/Delete on limited datasets" do
