@@ -25,6 +25,54 @@ describe "An SQLite database" do
     @db.sqlite_version.must_be_kind_of(Integer)
   end
   
+  it "should keep composite unique constraint when changing a column default" do
+    @db.create_table(:fk) do
+      Bignum :id, null: false, unique: true
+      Bignum :another_id, null: false
+      String :name, size: 50, null: false
+      String :test
+
+      unique [:another_id, :name], :name=>:fk_uidx
+    end
+    @db.alter_table(:fk) do
+      set_column_default :test, 'test'
+    end
+    @db[:fk].insert(:id=>1, :another_id=>2, :name=>'a')
+    @db[:fk].insert(:id=>2, :another_id=>3, :name=>'a')
+    @db[:fk].insert(:id=>3, :another_id=>2, :name=>'b')
+    proc{@db[:fk].insert(:id=>4, :another_id=>2, :name=>'a')}.must_raise Sequel::ConstraintViolation
+  end
+  
+  it "should keep composite primary key when changing a column default" do
+    @db.create_table(:fk) do
+      Bignum :id, null: false, unique: true
+      Bignum :another_id, null: false
+      String :name, size: 50, null: false
+      String :test
+
+      primary_key [:another_id, :name]
+    end
+    @db.alter_table(:fk) do
+      set_column_default :test, 'test'
+    end
+    @db[:fk].insert(:id=>1, :another_id=>2, :name=>'a')
+    @db[:fk].insert(:id=>2, :another_id=>3, :name=>'a')
+    @db[:fk].insert(:id=>3, :another_id=>2, :name=>'b')
+    proc{@db[:fk].insert(:id=>4, :another_id=>2, :name=>'a')}.must_raise Sequel::ConstraintViolation
+  end
+  
+  it "should allow setting current_timestamp_utc to keep CURRENT_* in UTC" do
+    begin
+      v = @db.current_timestamp_utc
+      @db.current_timestamp_utc = true
+      Time.parse(@db.get(Sequel::CURRENT_TIMESTAMP)).strftime('%Y%m%d%H%M').must_equal Time.now.utc.strftime('%Y%m%d%H%M')
+      Time.parse(@db.get(Sequel::CURRENT_DATE)).strftime('%Y%m%d').must_equal Time.now.utc.strftime('%Y%m%d')
+      Time.parse(@db.get(Sequel::CURRENT_TIME)).strftime('%H%M').must_equal Time.now.utc.strftime('%H%M')
+    ensure
+      @db.current_timestamp_utc = v
+    end
+  end
+  
   it "should support a use_timestamp_timezones setting" do
     @db.use_timestamp_timezones = true
     @db.create_table!(:fk){Time :time}
@@ -71,7 +119,9 @@ describe "An SQLite database" do
   
   it "should correctly parse the schema" do
     @db.create_table!(:fk) {timestamp :t}
-    @db.schema(:fk, :reload=>true).must_equal [[:t, {:type=>:datetime, :allow_null=>true, :default=>nil, :ruby_default=>nil, :db_type=>"timestamp", :primary_key=>false}]]
+    h = {:generated=>false, :type=>:datetime, :allow_null=>true, :default=>nil, :ruby_default=>nil, :db_type=>"timestamp", :primary_key=>false}
+    h.delete(:generated) if @db.sqlite_version < 33100
+    @db.schema(:fk, :reload=>true).must_equal [[:t, h]]
   end
 
   it "should handle and return BigDecimal values for numeric columns" do
@@ -86,6 +136,14 @@ describe "An SQLite database" do
     ds.map{|x| x[:d].to_s}.must_equal %w'NaN Infinity -Infinity'
     DB
   end
+
+  it "should support creating and parsing generated columns" do
+    @db.create_table!(:fk){Integer :a; Integer :b; Integer :c, :generated_always_as=>Sequel[:a] * 2 + :b + 1; Integer :d, :generated_always_as=>Sequel[:a] * 2 + :b + 2, :generated_type=>:stored; ; Integer :e, :generated_always_as=>Sequel[:a] * 2 + :b + 3, :generated_type=>:virtual}
+    @db[:fk].insert(:a=>100, :b=>10)
+    @db[:fk].select_order_map([:a, :b, :c, :d, :e]).must_equal [[100, 10, 211, 212, 213]]
+
+    @db.schema(:fk).map{|_,v| v[:generated]}.must_equal [false, false, true, true, true]
+  end if DB.sqlite_version >= 33100
 end
 
 describe "SQLite temporary views" do
