@@ -97,6 +97,14 @@ describe "DB#create_table" do
     @db.sqls.must_equal ['CREATE TABLE cats (a varchar(50), b text, c char(40), d time, e numeric(11, 2))']
   end
 
+  it "should use clob type for String columns if database uses clob for text" do
+    @db.extend(Module.new{private; def uses_clob_for_text?; true; end})
+    @db.create_table(:cats) do
+      String :b, :text=>true
+    end
+    @db.sqls.must_equal ['CREATE TABLE cats (b clob)']
+  end
+
   it "should allow the use of modifiers with ruby class types" do
     c = Class.new
     def c.name; 'Fixnum'; end
@@ -132,6 +140,12 @@ describe "DB#create_table" do
     end
     @db.sqls.must_equal ['CREATE TABLE cats (id integer PRIMARY KEY AUTOINCREMENT, a integer)']
 
+    @db.create_table(:cats) do
+      Integer :a
+      primary_key :id, :Bignum
+    end
+    @db.sqls.must_equal ['CREATE TABLE cats (id bigint PRIMARY KEY AUTOINCREMENT, a integer)']
+
     @db.create_table(:cats) do
       Integer :a
       primary_key :id, :keep_order=>true
@@ -163,6 +177,15 @@ describe "DB#create_table" do
     @db.sqls.must_equal ['CREATE TABLE cats (id varchar(255) NOT NULL, PRIMARY KEY (id))']
   end
 
+  it "should handle case where the primary key column cannot be found when adding NOT NULL constraint if database doesn't do it automatically" do
+    def @db.can_add_primary_key_constraint_on_nullable_columns?; false end
+    @db.create_table(:cats) do
+      String Sequel[:id]
+      primary_key [:id]
+    end
+    @db.sqls.must_equal ['CREATE TABLE cats (id varchar(255), PRIMARY KEY (id))']
+  end
+
   it "should handling splitting named column constraints into table constraints if unsupported" do
     def @db.supports_named_column_constraints?; false end
     @db.create_table(:cats) do
@@ -426,6 +449,13 @@ describe "DB#create_table" do
     @db.sqls.must_equal ["CREATE TABLE cats (id integer)", "CREATE INDEX cats_id_index ON cats (id)"]
   end
   
+  it "should accept inline index definition for qualified table" do
+    @db.create_table(Sequel[:sch][:cats]) do
+      integer :id, :index => true
+    end
+    @db.sqls.must_equal ["CREATE TABLE sch.cats (id integer)", "CREATE INDEX sch_cats_id_index ON sch.cats (id)"]
+  end
+  
   it "should accept inline index definition with a hash of options" do
     @db.create_table(:cats) do
       integer :id, :index => {:unique=>true}
@@ -824,6 +854,7 @@ describe "DB#create_table?" do
     @db.create_table?(:cats){|*a| Integer :a, :index=>true}
     @db.sqls.must_equal ['CREATE TABLE cats (a integer)', 'CREATE INDEX cats_a_index ON cats (a)']
 
+    @db.singleton_class.send(:alias_method, :table_exists?, :table_exists?)
     @db.define_singleton_method(:table_exists?){|a| true}
     @db.create_table?(:cats){|*a| Integer :a, :index=>true}
     @db.sqls.must_equal []
@@ -901,6 +932,7 @@ describe "DB#create_join_table?" do
     @db.create_join_table?(:cat_id=>:cats, :dog_id=>:dogs)
     @db.sqls.must_equal ['CREATE TABLE cats_dogs (cat_id integer NOT NULL REFERENCES cats, dog_id integer NOT NULL REFERENCES dogs, PRIMARY KEY (cat_id, dog_id))', 'CREATE INDEX cats_dogs_dog_id_cat_id_index ON cats_dogs (dog_id, cat_id)']
 
+    @db.singleton_class.send(:alias_method, :table_exists?, :table_exists?)
     @db.define_singleton_method(:table_exists?){|a| true}
     @db.create_join_table?(:cat_id=>:cats, :dog_id=>:dogs)
     @db.sqls.must_equal []
@@ -1344,6 +1376,27 @@ describe "DB#alter_table" do
       "ALTER TABLE cats ALTER COLUMN f SET DEFAULT 'g', ALTER COLUMN h TYPE integer, ADD CONSTRAINT i CHECK (a > 1), DROP CONSTRAINT j"]
   end
   
+  it "should handle operations that don't emit SQL when combining" do
+    @db.define_singleton_method(:supports_combining_alter_table_ops?){true}
+    @db.define_singleton_method(:combinable_alter_table_op?){|op| super(op) && (op[:op] != :rename_column || op[:name] == :d2)}
+    @db.define_singleton_method(:alter_table_op_sql){|t, op| super(t, op) unless op[:op] == :rename_column}
+    @db.alter_table(:cats) do
+      rename_column :d, :e
+      add_column :a, Integer
+      drop_column :b
+      set_column_not_null :c
+      rename_column :d2, :e2
+      add_index :e
+      set_column_default :f, 'g'
+      set_column_type :h, Integer
+      add_constraint(:i){a > 1}
+      drop_constraint :j
+    end
+    @db.sqls.must_equal ["ALTER TABLE cats ADD COLUMN a integer, DROP COLUMN b, ALTER COLUMN c SET NOT NULL",
+      "CREATE INDEX cats_e_index ON cats (e)",
+      "ALTER TABLE cats ALTER COLUMN f SET DEFAULT 'g', ALTER COLUMN h TYPE integer, ADD CONSTRAINT i CHECK (a > 1), DROP CONSTRAINT j"]
+  end
+  
 end
 
 describe "Database#create_table" do
@@ -1556,6 +1609,24 @@ describe "Database#create_view" do
     @db.sqls.must_equal ['DROP VIEW test', 'CREATE VIEW test AS SELECT a, b FROM items ORDER BY c']
   end
 
+  it "should handle create_or_replace_view when DROP VIEW raises a database error" do
+    def @db.drop_view(*) super; raise Sequel::DatabaseError end
+    @db.create_or_replace_view :test, @db[:items].select(:a, :b).order(:c)
+    @db.sqls.must_equal ['DROP VIEW test', 'CREATE VIEW test AS SELECT a, b FROM items ORDER BY c']
+  end
+
+  it "should raise in create_or_replace_view when DROP VIEW raises a disconnect error" do
+    def @db.drop_view(*) super; raise Sequel::DatabaseDisconnectError end
+    proc{@db.create_or_replace_view :test, @db[:items].select(:a, :b).order(:c)}.must_raise Sequel::DatabaseDisconnectError
+    @db.sqls.must_equal ['DROP VIEW test']
+  end
+
+  it "should raise in create_or_replace_view when DROP VIEW raises a connect error" do
+    def @db.drop_view(*) super; raise Sequel::DatabaseConnectionError end
+    proc{@db.create_or_replace_view :test, @db[:items].select(:a, :b).order(:c)}.must_raise Sequel::DatabaseConnectionError
+    @db.sqls.must_equal ['DROP VIEW test']
+  end
+
   it "should use CREATE OR REPLACE VIEW if such syntax is supported" do
     def @db.supports_create_or_replace_view?() true end
     @db.create_or_replace_view :test, @db[:items]
@@ -1662,6 +1733,15 @@ describe "Schema Parser" do
     c.must_equal ["y", {:schema=>"s", :dataset=>ds}]
   end
 
+  it "should raise error if asked to schema parse a dataset not involving a single table" do
+    @db.define_singleton_method(:schema_parse_table) do |t, opts|
+      [[:a, {:db_type=>t.to_s}]]
+    end
+    proc{@db.schema(@db.from)}.must_raise Sequel::Error
+    proc{@db.schema(@db.from(:x, :y))}.must_raise Sequel::Error
+    proc{@db.schema(@db.from(:x).cross_join(:y))}.must_raise Sequel::Error
+  end
+
   with_symbol_splitting "should provide options if given a table name with splittable symbols" do
     c = nil
     @db.define_singleton_method(:schema_parse_table) do |t, opts|
@@ -1799,6 +1879,8 @@ describe "Schema Parser" do
     @db.schema(:timestamp).first.last[:type].must_equal :datetime
     @db.schema(:"timestamp with time zone").first.last[:type].must_equal :datetime
     @db.schema(:"timestamp without time zone").first.last[:type].must_equal :datetime
+    @db.schema(:"timestamp(6)").first.last[:type].must_equal :datetime
+    @db.schema(:"timestamp(6) with time zone").first.last[:type].must_equal :datetime
     @db.schema(:time).first.last[:type].must_equal :time
     @db.schema(:"time with time zone").first.last[:type].must_equal :time
     @db.schema(:"time without time zone").first.last[:type].must_equal :time
