@@ -20,6 +20,8 @@ describe "constraint_validations extension" do
   before do
     @db = Sequel.mock
     @db.extend(Module.new{attr_writer :schema; def schema(table, *) execute("parse schema for #{table}"); @schema; end})
+    def @db.table_exists?(_) true; end
+    @db.singleton_class.send(:alias_method, :table_exists?, :table_exists?)
     @db.extension(:constraint_validations)
   end
 
@@ -36,13 +38,13 @@ describe "constraint_validations extension" do
 
   it "should allow dropping the sequel_constraint_validations table" do
     @db.drop_constraint_validations_table
-    @db.sqls.must_equal ["DROP TABLE sequel_constraint_validations"]
+    @db.sqls.must_equal ["DELETE FROM sequel_constraint_validations WHERE (table = 'sequel_constraint_validations')", "DROP TABLE sequel_constraint_validations"]
   end
 
   it "should allow dropping the sequel_constraint_validations table with a non-default table name" do
     @db.constraint_validations_table = :foo
     @db.drop_constraint_validations_table
-    @db.sqls.must_equal ["DROP TABLE foo"]
+    @db.sqls.must_equal ["DELETE FROM foo WHERE (table = 'foo')", "DROP TABLE foo"]
   end
 
   it "should allow dropping validations for a given table" do
@@ -50,6 +52,23 @@ describe "constraint_validations extension" do
     @db.sqls.must_equal ["DELETE FROM sequel_constraint_validations WHERE (table = 'foo')"]
   end
 
+  it "should drop validations for a given table when dropping the table" do
+    @db.drop_table(:foo)
+    @db.sqls.must_equal ["DELETE FROM sequel_constraint_validations WHERE (table = 'foo')", "DROP TABLE foo"]
+
+    @db.drop_table(:foo, :if_exists => true)
+    @db.sqls.must_equal ["DELETE FROM sequel_constraint_validations WHERE (table = 'foo')", "DROP TABLE IF EXISTS foo"]
+
+    @db.drop_table?(:foo)
+    @db.sqls.must_equal ["DELETE FROM sequel_constraint_validations WHERE (table = 'foo')", "DROP TABLE foo"]
+  end
+
+  it "should not drop validations for a given table if the constraint validations table does not exist" do
+    def @db.table_exists?(_) false; end
+    @db.drop_table(:foo)
+    @db.sqls.must_equal ["DROP TABLE foo"]
+  end
+
   it "should allow dropping validations for a given table and column" do
     @db.drop_constraint_validations_for(:table=>:foo, :column=>:bar)
     @db.sqls.must_equal ["DELETE FROM sequel_constraint_validations WHERE ((table = 'foo') AND (column = 'bar'))"]
@@ -78,6 +97,16 @@ describe "constraint_validations extension" do
     sqls.must_equal ["BEGIN", "COMMIT", "CREATE TABLE foo (name varchar(255), CHECK ((name IS NOT NULL) AND (trim(name) != '')))"]
   end
 
+  it "should allow adding constraint validations using generator directly" do
+    gen = @db.create_table_generator
+    gen.String :name
+    gen.validate{presence :name}
+    @db.create_table(:foo, :generator=>gen)
+    sqls = @db.sqls
+    parse_insert(sqls.slice!(1)).must_equal(:validation_type=>"presence", :column=>"name", :table=>"foo")
+    sqls.must_equal ["BEGIN", "COMMIT", "CREATE TABLE foo (name varchar(255), CHECK ((name IS NOT NULL) AND (trim(name) != '')))"]
+  end
+
   it "should allow adding constraint validations via alter_table validate" do
     @db.schema = [[:name, {:type=>:string}]]
     @db.alter_table(:foo){validate{presence :name}}
@@ -86,6 +115,12 @@ describe "constraint_validations extension" do
     sqls.must_equal ["parse schema for foo", "BEGIN", "COMMIT", "ALTER TABLE foo ADD CHECK ((name IS NOT NULL) AND (trim(name) != ''))"]
   end
 
+  it "should allow altering the table without validation" do
+    @db.schema = [[:name, {:type=>:string}]]
+    @db.alter_table(:foo){rename_column :a, :b}
+    @db.sqls.must_equal ["ALTER TABLE foo RENAME COLUMN a TO b"]
+  end
+
   it "should handle :message option when adding validations" do
     @db.create_table(:foo){String :name; validate{presence :name, :message=>'not there'}}
     sqls = @db.sqls
@@ -107,12 +142,38 @@ describe "constraint_validations extension" do
     sqls.must_equal ["BEGIN", "COMMIT", "CREATE TABLE foo (name varchar(255), CONSTRAINT cons CHECK ((name IS NOT NULL) AND (trim(name) != '')))"]
   end
 
-  it "should handle multiple columns when adding validations" do
+  it "should handle multiple string columns when adding presence validations" do
     @db.create_table(:foo){String :name; String :bar; validate{presence [:name, :bar]}}
     sqls = @db.sqls
     parse_insert(sqls.slice!(1)).must_equal(:validation_type=>"presence", :column=>"name", :table=>"foo")
     parse_insert(sqls.slice!(1)).must_equal(:validation_type=>"presence", :column=>"bar", :table=>"foo")
-    sqls.must_equal ["BEGIN", "COMMIT", "CREATE TABLE foo (name varchar(255), bar varchar(255), CHECK ((name IS NOT NULL) AND (bar IS NOT NULL) AND (trim(name) != '') AND (trim(bar) != '')))"]
+    sqls.must_equal ["BEGIN", "COMMIT", "CREATE TABLE foo (name varchar(255), bar varchar(255), CHECK ((name IS NOT NULL) AND (trim(name) != '') AND (bar IS NOT NULL) AND (trim(bar) != '')))"]
+  end
+
+  it "should handle multiple string columns when adding presence validations with :allow_nil" do
+    @db.create_table(:foo){String :name; String :bar; validate{presence [:name, :bar], :allow_nil=>true}}
+    sqls = @db.sqls
+    parse_insert(sqls.slice!(1)).must_equal(:validation_type=>"presence", :column=>"name", :table=>"foo", :allow_nil=>'t')
+    parse_insert(sqls.slice!(1)).must_equal(:validation_type=>"presence", :column=>"bar", :table=>"foo", :allow_nil=>'t')
+    sqls.must_equal ["BEGIN", "COMMIT", "CREATE TABLE foo (name varchar(255), bar varchar(255), CHECK (((name IS NULL) OR (trim(name) != '')) AND ((bar IS NULL) OR (trim(bar) != ''))))"]
+  end
+
+  it "should handle multiple string columns when adding presence validations" do
+    @db.create_table(:foo){String :name; Integer :x; String :bar; validate{presence [:name, :x, :bar]}}
+    sqls = @db.sqls
+    parse_insert(sqls.slice!(1)).must_equal(:validation_type=>"presence", :column=>"name", :table=>"foo")
+    parse_insert(sqls.slice!(1)).must_equal(:validation_type=>"presence", :column=>"x", :table=>"foo")
+    parse_insert(sqls.slice!(1)).must_equal(:validation_type=>"presence", :column=>"bar", :table=>"foo")
+    sqls.must_equal ["BEGIN", "COMMIT", "CREATE TABLE foo (name varchar(255), x integer, bar varchar(255), CHECK ((name IS NOT NULL) AND (trim(name) != '') AND (bar IS NOT NULL) AND (trim(bar) != '') AND (x IS NOT NULL)))"]
+  end
+
+  it "should handle multiple string columns when adding presence validations with :allow_nil" do
+    @db.create_table(:foo){String :name; Integer :x; String :bar; validate{presence [:name, :x, :bar], :allow_nil=>true}}
+    sqls = @db.sqls
+    parse_insert(sqls.slice!(1)).must_equal(:validation_type=>"presence", :column=>"name", :table=>"foo", :allow_nil=>'t')
+    parse_insert(sqls.slice!(1)).must_equal(:validation_type=>"presence", :column=>"x", :table=>"foo", :allow_nil=>'t')
+    parse_insert(sqls.slice!(1)).must_equal(:validation_type=>"presence", :column=>"bar", :table=>"foo", :allow_nil=>'t')
+    sqls.must_equal ["BEGIN", "COMMIT", "CREATE TABLE foo (name varchar(255), x integer, bar varchar(255), CHECK (((name IS NULL) OR (trim(name) != '')) AND ((bar IS NULL) OR (trim(bar) != ''))))"]
   end
 
   it "should handle presence validation on non-String columns" do
@@ -132,6 +193,7 @@ describe "constraint_validations extension" do
     @db = Sequel.mock(:host=>'oracle')
     @db.extend_datasets do
       def quote_identifiers?; false end
+      private
       def input_identifier(v) v.to_s end
     end
     @db.extension(:constraint_validations)
