@@ -116,9 +116,6 @@ describe "NestedAttributes plugin" do
       end
     end
     @Concert.class_eval do
-      define_method :_insert do
-        insert = values.dup
-      end
       def before_create # Have to define the CPK somehow.
         self.tour = 'To'
         self.date = '2004-04-05'
@@ -128,6 +125,10 @@ describe "NestedAttributes plugin" do
         super
         self.artist_id = 3
       end
+      private
+      define_method :_insert do
+        insert = values.dup
+      end
     end
 
     c = @Concert.new(:playlist=>'Pl')
@@ -188,14 +189,15 @@ describe "NestedAttributes plugin" do
   it "should support creating new objects with composite primary keys" do
     insert = nil
     @Concert.class_eval do
-      define_method :_insert do
-        insert = values.dup
-      end
       def before_create # Have to define the CPK somehow.
         self.tour = 'To'
         self.date = '2004-04-05'
         super
       end
+      private
+      define_method :_insert do
+        insert = values.dup
+      end
     end
     a = @Artist.new({:name=>'Ar', :concerts_attributes=>[{:playlist=>'Pl'}]})
     @db.sqls.must_equal []
@@ -209,6 +211,7 @@ describe "NestedAttributes plugin" do
     insert = nil
     @Album.class_eval do
       unrestrict_primary_key
+      private
       define_method :_insert do
         insert = values.dup
       end
@@ -224,6 +227,7 @@ describe "NestedAttributes plugin" do
     insert = nil
     @Artist.nested_attributes :concerts, :unmatched_pk=>:create
     @Concert.class_eval do
+      private
       define_method :_insert do
         insert = values.dup
       end
@@ -702,6 +706,7 @@ describe "NestedAttributes plugin" do
     @Album.nested_attributes :tags, :remove=>true, :unmatched_pk=>:ignore
     objs = []
     @Album.class_eval do
+      private
       define_method(:nested_attributes_create){|*a| objs << [super(*a), :create]}
       define_method(:nested_attributes_remove){|*a| objs << [super(*a), :remove]}
       define_method(:nested_attributes_update){|*a| objs << [super(*a), :update]}
@@ -803,13 +808,26 @@ describe "NestedAttributes plugin" do
     proc{@Album.load(:id=>10, :name=>'Al').set_nested_attributes(:tags2, [{:id=>30, :name=>'T2', :number=>3}], :fields=>[:name])}.must_raise(Sequel::Error)
   end
 
-  it "should have set_nested_attributes method raise error if called with an association that doesn't support nested attributes" do
+  it "should have set_nested_attributes method raise error if there is no existing object with the given primary key" do
     @Tag.columns :id, :name, :number
     proc{@Album.load(:id=>10, :name=>'Al').set_nested_attributes(:tags, [{:id=>30, :name=>'T2', :number=>3}], :fields=>[:name])}.must_raise(Sequel::Error)
   end
 
-  it "should not allow modifying ensted attributes after freezing" do
+  it "should have set_nested_attributes method raise error if called with an association that doesn't support nested attributes" do
+    @Album.many_to_many :tags, :class=>@Tag, :left_key=>:album_id, :right_key=>:tag_id, :join_table=>:at
+    proc{@Album.load(:id=>10, :name=>'Al').set_nested_attributes(:tags, [{:id=>30, :name=>'T2', :number=>3}], :fields=>[:name])}.must_raise(Sequel::Error)
+  end
+
+  it "should not allow adding nested attributes after freezing" do
     @Artist.freeze
-    proc{@Artist.nested_attributes :albums}.must_raise RuntimeError, TypeError
+    proc{@Artist.nested_attributes :albums}.must_raise RuntimeError
+  end
+
+  it "should not adding modifying nested attributes after freezing if none were present before" do
+    @c = Class.new(Sequel::Model(@db))
+    @c.plugin :nested_attributes
+    @Artist = Class.new(@c).set_dataset(:artists)
+    @Artist.freeze
+    proc{@Artist.nested_attributes :albums}.must_raise RuntimeError
   end
 end
