@@ -47,6 +47,17 @@ describe "Sequel::Schema::CreateTableGenerator dump methods" do
     g.columns.must_equal g2.columns
   end
 
+  it "should respect :keep_order option to primary_key with primary key type" do
+    g = @g.new(@d) do
+      Integer :a
+      primary_key :c, :keep_order=>true, :type=>:Bignum
+    end
+    g2 = @g.new(@d) do
+      instance_eval(g.dump_columns, __FILE__, __LINE__)
+    end
+    g.columns.must_equal g2.columns
+  end
+
   it "should allow dumping indexes as separate add_index and drop_index methods" do
     g = @g.new(@d) do
       index :a
@@ -76,6 +87,7 @@ describe "Sequel::Database dump methods" do
   before do
     @d = Sequel::Database.new.extension(:schema_dumper)
     def @d.tables(o) o[:schema] ? [o[:schema]] : [:t1, :t2] end
+    @d.singleton_class.send(:alias_method, :tables, :tables)
     def @d.schema(t, *o)
       v = case t
       when :t1, 't__t1', Sequel.identifier(:t__t1)
@@ -89,6 +101,10 @@ describe "Sequel::Database dump methods" do
          [:c1, {:db_type=>'integer', :primary_key=>true, :auto_increment=>true, :allow_null=>false}]]
       when :t5
         [[:c1, {:db_type=>'blahblah', :allow_null=>true}]]
+      when :t6
+        [[:c1, {:db_type=>'integer', :allow_null=>false, :generated=>true, :default=>'1', :ruby_default=>1}]]
+      when :t7
+        [[:c1, {:db_type=>'integer', :allow_null=>true, :generated=>true, :default=>'(a + b)', :ruby_default=>nil}]]
       end
 
       if o.first.is_a?(Hash) && o.first[:schema]
@@ -97,6 +113,7 @@ describe "Sequel::Database dump methods" do
 
       v
     end
+    @d.singleton_class.send(:alias_method, :schema, :schema)
   end
 
   it "should support dumping table with :schema option" do
@@ -207,13 +224,20 @@ OUTPUT
     @d.dump_table_schema(:t1).must_equal "create_table(:t1) do\n  Integer :c1\n  Integer :c2\n  \n  foreign_key [:c1, :c2], :t2, :key=>[:c3, :c4]\nend"
   end
 
+  it "should use a composite foreign_key calls with options" do
+    def @d.schema(*s) [[:c1, {:db_type=>'integer'}], [:c2, {:db_type=>'integer'}]] end
+    def @d.supports_foreign_key_parsing?; true end
+    def @d.foreign_key_list(*s) [{:columns=>[:c1, :c2], :table=>:t2, :key=>[:c3, :c4], :on_delete=>:no_action, :on_update=>:no_action, :deferrable=>true}] end
+    @d.dump_table_schema(:t1).must_equal "create_table(:t1) do\n  Integer :c1\n  Integer :c2\n  \n  foreign_key [:c1, :c2], :t2, :key=>[:c3, :c4], :deferrable=>true\nend"
+  end
+
   it "should include index information if available" do
     def @d.supports_index_parsing?; true end
     def @d.indexes(t)
       {:i1=>{:columns=>[:c1], :unique=>false},
-       :t1_c2_c1_index=>{:columns=>[:c2, :c1], :unique=>true}}
+       :t1_c2_c1_index=>{:columns=>[:c2, :c1], :unique=>true, :deferrable=>true}}
     end
-    @d.dump_table_schema(:t1).must_equal "create_table(:t1, :ignore_index_errors=>true) do\n  primary_key :c1\n  String :c2, :size=>20\n  \n  index [:c1], :name=>:i1\n  index [:c2, :c1], :unique=>true\nend"
+    @d.dump_table_schema(:t1).must_equal "create_table(:t1, :ignore_index_errors=>true) do\n  primary_key :c1\n  String :c2, :size=>20\n  \n  index [:c1], :name=>:i1\n  index [:c2, :c1], :unique=>true, :deferrable=>true\nend"
   end
 
   it "should support dumping the whole database as a migration with a :schema option" do
@@ -693,6 +717,22 @@ END_MIG
     @d.dump_table_schema(:t5).must_equal "create_table(:t5) do\n  String :c1\nend"
   end
 
+  it "should not include defaults for generated columns even if parsed" do
+    @d.dump_table_schema(:t6).must_equal "create_table(:t6) do\n  Integer :c1, :null=>false\nend"
+    @d.dump_table_schema(:t7).must_equal "create_table(:t7) do\n  Integer :c1\nend"
+  end
+
+  it "should not include defaults for generated columns even if parsed when using :same_db option" do
+    @d.dump_table_schema(:t6, :same_db=>true).must_equal "create_table(:t6) do\n  column :c1, \"integer\", :null=>false\nend"
+    @d.dump_table_schema(:t7, :same_db=>true).must_equal "create_table(:t7) do\n  column :c1, \"integer\"\nend"
+  end
+
+  it "should create generated column when using :same_db option on PostgreSQL" do
+    def @d.database_type; :postgres end
+    @d.dump_table_schema(:t6, :same_db=>true).must_equal "create_table(:t6) do\n  column :c1, \"integer\", :generated_always_as=>Sequel::LiteralString.new(\"1\"), :null=>false\nend"
+    @d.dump_table_schema(:t7, :same_db=>true).must_equal "create_table(:t7) do\n  column :c1, \"integer\", :generated_always_as=>Sequel::LiteralString.new(\"(a + b)\")\nend"
+  end
+
   it "should convert many database types to ruby types" do
     def @d.schema(t, *o)
       types = %w"mediumint smallint int integer mediumint(6) smallint(7) int(8) integer(9)
@@ -706,7 +746,7 @@ END_MIG
       %w"nvarchar ntext smalldatetime smallmoney binary varbinary nchar" +
       ["timestamp(6) without time zone", "timestamp(6) with time zone", 'mediumint(10) unsigned', 'int(9) unsigned',
        'int(10) unsigned', "int(12) unsigned", 'bigint unsigned', 'tinyint(3) unsigned', 'identity', 'int identity'] +
-      %w"integer(10) bit bool"
+      %w"integer(10) bit bool" + ["decimal(7, 2) unsigned", "real unsigned"]
       i = 0
       types.map{|x| [:"c#{i+=1}", {:db_type=>x, :allow_null=>true}]}
     end
@@ -786,6 +826,8 @@ create_table(:x) do
   Integer :c72
   TrueClass :c73
   TrueClass :c74
+  BigDecimal :c75, :size=>[7, 2]
+  Float :c76
   
   check Sequel::SQL::BooleanExpression.new(:>=, Sequel::SQL::Identifier.new(:c64), 0)
   check Sequel::SQL::BooleanExpression.new(:>=, Sequel::SQL::Identifier.new(:c65), 0)
@@ -793,6 +835,8 @@ create_table(:x) do
   check Sequel::SQL::BooleanExpression.new(:>=, Sequel::SQL::Identifier.new(:c67), 0)
   check Sequel::SQL::BooleanExpression.new(:>=, Sequel::SQL::Identifier.new(:c68), 0)
   check Sequel::SQL::BooleanExpression.new(:>=, Sequel::SQL::Identifier.new(:c69), 0)
+  check Sequel::SQL::BooleanExpression.new(:>=, Sequel::SQL::Identifier.new(:c75), 0)
+  check Sequel::SQL::BooleanExpression.new(:>=, Sequel::SQL::Identifier.new(:c76), 0)
 end
 END_MIG
   end
@@ -800,13 +844,15 @@ END_MIG
   it "should convert mysql types to ruby types" do
     def @d.schema(t, *o)
       i = 0
-      ['float unsigned', 'double(15,2)', 'double(7,1) unsigned'].map{|x| [:"c#{i+=1}", {:db_type=>x, :allow_null=>true}]}
+      ['float unsigned', 'double(15,2)', 'double(7,1) unsigned', 'tinyint', 'char(3)'].map{|x| [:"c#{i+=1}", {:db_type=>x, :allow_null=>true, :type=>:boolean, :size=>10}]}
     end
     @d.dump_table_schema(:x).must_equal((<<END_MIG).chomp)
 create_table(:x) do
   Float :c1
   Float :c2
   Float :c3
+  TrueClass :c4
+  String :c5, :size=>3, :fixed=>true
   
   check Sequel::SQL::BooleanExpression.new(:>=, Sequel::SQL::Identifier.new(:c1), 0)
   check Sequel::SQL::BooleanExpression.new(:>=, Sequel::SQL::Identifier.new(:c3), 0)
@@ -833,6 +879,7 @@ END_MIG
     def @d.database_type; :mysql end
     def @d.schema(*s) [[:c1, {:db_type=>'timestamp', :primary_key=>true, :allow_null=>true}]] end
     @d.dump_table_schema(:t3, :same_db=>true).must_equal "create_table(:t3) do\n  column :c1, \"timestamp\", :null=>true\n  \n  primary_key [:c1]\nend"
+    @d.singleton_class.send(:alias_method, :schema, :schema)
 
     def @d.schema(*s) [[:c1, {:db_type=>'timestamp', :primary_key=>true, :allow_null=>false}]] end
     @d.dump_table_schema(:t3, :same_db=>true).must_equal "create_table(:t3) do\n  column :c1, \"timestamp\", :null=>false\n  \n  primary_key [:c1]\nend"
