@@ -1723,7 +1723,12 @@ describe "SQL Extract Function" do
     @ds.get{a.extract(:day)}.must_equal t.day
     @ds.get{a.extract(:hour)}.must_equal t.hour
     @ds.get{a.extract(:minute)}.must_equal t.min
-    @ds.get{a.extract(:second)}.to_i.must_equal t.sec
+    sec = @ds.get{a.extract(:second)}.to_i
+    if t.usec >= 999500
+      sec.must_be_close_to 1, t.sec
+    else
+      sec.to_i.must_equal t.sec
+    end
   end
 end
 
@@ -1935,7 +1940,7 @@ describe "Dataset identifier methods" do
   end
 end if IDENTIFIER_MANGLING
 
-if DB.dataset.supports_modifying_joins?
+if DB.dataset.supports_updating_joins? || DB.dataset.supports_deleting_joins?
   describe "Modifying joined datasets" do
     before do
       @db = DB
@@ -1954,20 +1959,24 @@ if DB.dataset.supports_modifying_joins?
       @db.drop_table?(:a, :b, :c)
     end
     
-    it "#update should allow updating joined datasets" do
-      @ds.update(:a=>10)
-      @ds.all.must_equal [{:c=>5, :b=>2, :a=>10, :d=>2, :e=>5, :f=>6}]
-      @db[:a].order(:a).all.must_equal [{:a=>3, :d=>4}, {:a=>10, :d=>2}]
-      @db[:b].order(:b).all.must_equal [{:b=>2, :e=>5}, {:b=>4, :e=>7}]
-      @db[:c].order(:c).all.must_equal [{:c=>5, :f=>6}, {:c=>7, :f=>8}]
+    if DB.dataset.supports_updating_joins?
+      it "#update should allow updating joined datasets" do
+        @ds.update(:a=>10)
+        @ds.all.must_equal [{:c=>5, :b=>2, :a=>10, :d=>2, :e=>5, :f=>6}]
+        @db[:a].order(:a).all.must_equal [{:a=>3, :d=>4}, {:a=>10, :d=>2}]
+        @db[:b].order(:b).all.must_equal [{:b=>2, :e=>5}, {:b=>4, :e=>7}]
+        @db[:c].order(:c).all.must_equal [{:c=>5, :f=>6}, {:c=>7, :f=>8}]
+      end
     end
     
-    it "#delete should allow deleting from joined datasets" do
-      @ds.delete
-      @ds.all.must_equal []
-      @db[:a].order(:a).all.must_equal [{:a=>3, :d=>4}]
-      @db[:b].order(:b).all.must_equal [{:b=>2, :e=>5}, {:b=>4, :e=>7}]
-      @db[:c].order(:c).all.must_equal [{:c=>5, :f=>6}, {:c=>7, :f=>8}]
+    if DB.dataset.supports_deleting_joins?
+      it "#delete should allow deleting from joined datasets" do
+        @ds.delete
+        @ds.all.must_equal []
+        @db[:a].order(:a).all.must_equal [{:a=>3, :d=>4}]
+        @db[:b].order(:b).all.must_equal [{:b=>2, :e=>5}, {:b=>4, :e=>7}]
+        @db[:c].order(:c).all.must_equal [{:c=>5, :f=>6}, {:c=>7, :f=>8}]
+      end
     end
   end
 end
@@ -2035,3 +2044,41 @@ describe "Dataset replace" do
     @d.all.must_equal [{:id => 111, :value => 333}]
   end
 end if DB.dataset.supports_replace?
+
+describe "Concurrent access" do
+  before do
+    @ds = DB.select(Sequel[1].as(:v)).union(DB.select 2)
+  end
+  after do
+    DB.disconnect
+  end
+
+  it "should support multiple threads" do
+    threads = 4.times.map do
+      q = Queue.new
+      q2 = Queue.new
+      [q, q2, Thread.new{@ds.each{|r| q.push(r[:v]); q2.pop}}]
+    end
+    threads.each{|q,| q.pop.must_equal 1}
+    threads.each{|_,q2| q2.push nil}
+    threads.each{|q,| q.pop.must_equal 2}
+    threads.each{|_,q2| q2.push nil}
+    threads.each{|_,_,t| t.join}
+  end
+
+  if ENV["SEQUEL_FIBER_CONCURRENCY"]
+    it "should support multiple enumerators" do
+      enums = 4.times.map{@ds.to_enum}
+      enums.each{|e| e.next[:v].must_equal 1}
+      enums.each{|e| e.next[:v].must_equal 2}
+      enums.each{|e| proc{e.next}.must_raise StopIteration}
+    end
+
+    it "should support multiple fibers" do
+      fibers = 4.times.map{Fiber.new{@ds.each{|r| Fiber.yield r[:v]}; 3}}
+      fibers.each{|f| f.resume.must_equal 1}
+      fibers.each{|f| f.resume.must_equal 2}
+      fibers.each{|f| f.resume.must_equal 3}
+    end
+  end
+end if [:threaded, :sharded_threaded].include?(DB.pool.pool_type) && DB.pool.max_size >= 4 && DB.database_type != :derby && DB.database_type != :sqlanywhere
