@@ -1,50 +1,54 @@
-require File.join(File.dirname(File.expand_path(__FILE__)), 'spec_helper.rb')
+require_relative "spec_helper"
 
 describe "Database schema parser" do
-  before do
-    @iom = DB.identifier_output_method
-    @iim = DB.identifier_input_method
-    @qi = DB.quote_identifiers?
-  end
   after do
-    DB.identifier_output_method = @iom
-    DB.identifier_input_method = @iim
-    DB.quote_identifiers = @qi
     DB.drop_table?(:items)
   end
 
-  it "should handle a database with a identifier methods" do
-    DB.identifier_output_method = :reverse
-    DB.identifier_input_method = :reverse
-    DB.quote_identifiers = true
-    DB.create_table!(:items){Integer :number}
-    begin
-      DB.schema(:items, :reload=>true).must_be_kind_of(Array)
-      DB.schema(:items, :reload=>true).first.first.must_equal :number
-    ensure 
-      DB.drop_table(:items)
+  describe "with identifier mangling" do
+    before do
+      @iom = DB.identifier_output_method
+      @iim = DB.identifier_input_method
+      @qi = DB.quote_identifiers?
+    end
+    after do
+      DB.identifier_output_method = @iom
+      DB.identifier_input_method = @iim
+      DB.quote_identifiers = @qi
     end
-  end
 
-  it "should handle a dataset with identifier methods different than the database's" do
-    DB.identifier_output_method = :reverse
-    DB.identifier_input_method = :reverse
-    DB.quote_identifiers = true
-    DB.create_table!(:items){Integer :number}
-    DB.identifier_output_method = @iom
-    DB.identifier_input_method = @iim
-    ds = DB[:items]
-    ds.identifier_output_method = :reverse
-    ds.identifier_input_method = :reverse
-    begin
-      DB.schema(ds, :reload=>true).must_be_kind_of(Array)
-      DB.schema(ds, :reload=>true).first.first.must_equal :number
-    ensure 
+    it "should handle a database with a identifier methods" do
       DB.identifier_output_method = :reverse
       DB.identifier_input_method = :reverse
-      DB.drop_table(:items)
+      DB.quote_identifiers = true
+      DB.create_table!(:items){Integer :number}
+      begin
+        DB.schema(:items, :reload=>true).must_be_kind_of(Array)
+        DB.schema(:items, :reload=>true).first.first.must_equal :number
+      ensure 
+      end
     end
-  end
+
+    it "should handle a dataset with identifier methods different than the database's" do
+      DB.identifier_output_method = :reverse
+      DB.identifier_input_method = :reverse
+      DB.quote_identifiers = true
+      DB.create_table!(:items){Integer :number}
+      DB.identifier_output_method = @iom
+      DB.identifier_input_method = @iim
+      ds = DB[:items].
+        with_identifier_output_method(:reverse).
+        with_identifier_input_method(:reverse)
+      begin
+        DB.schema(ds, :reload=>true).must_be_kind_of(Array)
+        DB.schema(ds, :reload=>true).first.first.must_equal :number
+      ensure 
+        DB.identifier_output_method = :reverse
+        DB.identifier_input_method = :reverse
+        DB.drop_table(:items)
+      end
+    end
+  end if IDENTIFIER_MANGLING && !DB.frozen?
 
   it "should not issue an sql query if the schema has been loaded unless :reload is true" do
     DB.create_table!(:items){Integer :number}
@@ -106,7 +110,7 @@ describe "Database schema parser" do
 
   it "should parse defaults from the schema properly" do
     DB.create_table!(:items){Integer :number}
-    DB.schema(:items).first.last[:ruby_default].must_equal nil
+    DB.schema(:items).first.last[:ruby_default].must_be_nil
     DB.create_table!(:items){Integer :number, :default=>0}
     DB.schema(:items).first.last[:ruby_default].must_equal 0
     DB.create_table!(:items){String :a, :default=>"blah"}
@@ -115,7 +119,7 @@ describe "Database schema parser" do
 
   it "should make :default nil for a NULL default" do
     DB.create_table!(:items){Integer :number}
-    DB.schema(:items).first.last[:default].must_equal nil
+    DB.schema(:items).first.last[:default].must_be_nil
     DB.create_table!(:items){Integer :number, :default=>0}
     DB.schema(:items).first.last[:default].wont_equal nil
   end
@@ -125,7 +129,7 @@ describe "Database schema parser" do
     DB.schema(:items).first.last[:ruby_default].must_equal Sequel::CURRENT_TIMESTAMP
   end
 
-  cspecify "should parse current date defaults from the schema properly", :mysql, :oracle do
+  cspecify "should parse current date defaults from the schema properly", [proc{|db| !db.mariadb? || db.server_version <= 100200}, :mysql], :oracle do
     DB.create_table!(:items){Date :a, :default=>Sequel::CURRENT_DATE}
     DB.schema(:items).first.last[:ruby_default].must_equal Sequel::CURRENT_DATE
   end
@@ -185,23 +189,25 @@ describe "Database index parsing" do
   end
 
   it "should parse indexes into a hash" do
-    # Delete :deferrable entry, since not all adapters implement it
-    f = lambda{h = DB.indexes(:items); h.values.each{|h2| h2.delete(:deferrable)}; h}
+    [:items, Sequel.identifier(:items)].each do |table|
+      # Delete :deferrable entry, since not all adapters implement it
+      f = lambda{h = DB.indexes(table); h.values.each{|h2| h2.delete(:deferrable)}; h}
 
-    DB.create_table!(:items){Integer :n; Integer :a}
-    f.call.must_equal({})
-    DB.add_index(:items, :n)
-    f.call.must_equal(:items_n_index=>{:columns=>[:n], :unique=>false})
-    DB.drop_index(:items, :n)
-    f.call.must_equal({})
-    DB.add_index(:items, :n, :unique=>true, :name=>:blah_blah_index)
-    f.call.must_equal(:blah_blah_index=>{:columns=>[:n], :unique=>true})
-    DB.add_index(:items, [:n, :a])
-    f.call.must_equal(:blah_blah_index=>{:columns=>[:n], :unique=>true}, :items_n_a_index=>{:columns=>[:n, :a], :unique=>false})
-    DB.drop_index(:items, :n, :name=>:blah_blah_index)
-    f.call.must_equal(:items_n_a_index=>{:columns=>[:n, :a], :unique=>false})
-    DB.drop_index(:items, [:n, :a])
-    f.call.must_equal({})
+      DB.create_table!(table){Integer :n; Integer :a}
+      f.call.must_equal({})
+      DB.add_index(table, :n)
+      f.call.must_equal(:items_n_index=>{:columns=>[:n], :unique=>false})
+      DB.drop_index(table, :n)
+      f.call.must_equal({})
+      DB.add_index(table, :n, :unique=>true, :name=>:blah_blah_index)
+      f.call.must_equal(:blah_blah_index=>{:columns=>[:n], :unique=>true})
+      DB.add_index(table, [:n, :a])
+      f.call.must_equal(:blah_blah_index=>{:columns=>[:n], :unique=>true}, :items_n_a_index=>{:columns=>[:n, :a], :unique=>false})
+      DB.drop_index(table, :n, :name=>:blah_blah_index)
+      f.call.must_equal(:items_n_a_index=>{:columns=>[:n, :a], :unique=>false})
+      DB.drop_index(table, [:n, :a])
+      f.call.must_equal({})
+    end
   end
   
   it "should not include a primary key index" do
@@ -211,7 +217,7 @@ describe "Database index parsing" do
     DB.indexes(:items).must_equal({})
   end
 
-  cspecify "should not include partial indexes", :sqlite do
+  cspecify "should not include partial indexes", [proc{|db| db.sqlite_version < 30808}, :sqlite] do
     DB.create_table!(:items){Integer :n; Integer :a; index :n, :where=>proc{n > 10}}
     DB.indexes(:items).must_equal({})
   end if DB.supports_partial_indexes?
@@ -383,6 +389,26 @@ describe "Database schema modifiers" do
     @db.table_exists?(:items).must_equal false
   end if DB.supports_transactional_ddl?
   
+  it "should handle errors creating indexes when ignoring index errors" do
+    @db.drop_table?(:items)
+    @db.transaction do
+      @db.create_table(:items, :ignore_index_errors=>true) do
+        Integer :n1
+        Integer :n2
+        index :n1, :name=>'items_n1'
+        index :foo, :name=>'items_f'
+        index :n2, :name=>'items_n2'
+        index :bar, :name=>'items_g'
+      end
+    end
+    @db.table_exists?(:items).must_equal true
+    indexes = @db.indexes(:items).keys
+    indexes.must_include :items_n1
+    indexes.must_include :items_n2
+    indexes.wont_include :items_f
+    indexes.wont_include :items_g
+  end if DB.supports_transactional_ddl? && DB.database_type != :mssql
+  
   describe "join tables" do
     after do
       @db.drop_join_table(:cat_id=>:cats, :dog_id=>:dogs) if @db.table_exists?(:cats_dogs)
@@ -515,6 +541,27 @@ describe "Database schema modifiers" do
     proc{@ds.insert(:id=>nil)}.must_raise(Sequel::NotNullConstraintViolation, Sequel::ConstraintViolation, Sequel::DatabaseError)
   end
 
+  it "should not allow NULLs when adding a primary key column" do
+    @db.create_table!(:items){String :foo}
+    @db.alter_table(:items){add_column :id, String, :primary_key=>true, :default=>'a'}
+    proc{@ds.insert(:id=>nil)}.must_raise(Sequel::NotNullConstraintViolation, Sequel::ConstraintViolation, Sequel::DatabaseError)
+  end
+
+  it "should not allow NULLs when creating table with primary key constraint" do
+    @db.create_table!(:items){String :id1; String :id2; primary_key [:id1, :id2]}
+    proc{@ds.insert(:id1=>nil, :id2=>nil)}.must_raise(Sequel::NotNullConstraintViolation, Sequel::ConstraintViolation, Sequel::DatabaseError)
+    proc{@ds.insert(:id1=>nil, :id2=>'1')}.must_raise(Sequel::NotNullConstraintViolation, Sequel::ConstraintViolation, Sequel::DatabaseError)
+    proc{@ds.insert(:id1=>'1', :id2=>nil)}.must_raise(Sequel::NotNullConstraintViolation, Sequel::ConstraintViolation, Sequel::DatabaseError)
+  end
+
+  it "should not allow NULLs when adding a primary key constraint" do
+    @db.create_table!(:items){String :id1; String :id2}
+    @db.alter_table(:items){add_primary_key [:id1, :id2]}
+    proc{@ds.insert(:id1=>nil, :id2=>nil)}.must_raise(Sequel::NotNullConstraintViolation, Sequel::ConstraintViolation, Sequel::DatabaseError)
+    proc{@ds.insert(:id1=>nil, :id2=>'1')}.must_raise(Sequel::NotNullConstraintViolation, Sequel::ConstraintViolation, Sequel::DatabaseError)
+    proc{@ds.insert(:id1=>'1', :id2=>nil)}.must_raise(Sequel::NotNullConstraintViolation, Sequel::ConstraintViolation, Sequel::DatabaseError)
+  end
+
   it "should rename columns correctly" do
     @db.create_table!(:items){Integer :id}
     @ds.insert(:id=>10)
@@ -565,7 +612,7 @@ describe "Database schema modifiers" do
     @ds.all.must_equal [{:id2=>10}]
   end
 
-  cspecify "should set column NULL/NOT NULL correctly", [:jdbc, :db2], [:db2] do
+  cspecify "should set column NULL/NOT NULL correctly", [:jdbc, :db2] do
     @db.create_table!(:items, :engine=>:InnoDB){Integer :id}
     @ds.insert(:id=>10)
     @db.alter_table(:items){set_column_allow_null :id, false}
@@ -587,7 +634,36 @@ describe "Database schema modifiers" do
     @ds.all.must_equal [{:id=>10}, {:id=>20}]
   end
 
-  cspecify "should set column types correctly", [:jdbc, :db2], [:db2], :oracle do
+  it "should set column defaults correctly if column has existing default" do
+    @db.create_table!(:items){Integer :id, :default=>10}
+    @ds.insert
+    @ds.all.must_equal [{:id=>10}]
+    @db.alter_table(:items){set_column_default :id, 20}
+    @ds.insert
+    @ds.all.must_equal [{:id=>10}, {:id=>20}]
+  end
+
+  it "should set column defaults to nil correctly" do
+    @db.create_table!(:items){Integer :id}
+    @ds.insert(:id=>10)
+    @db.alter_table(:items){set_column_default :id, nil}
+    @db.schema(:items, :reload=>true).map{|x| x.first}.must_equal [:id]
+    @ds.columns!.must_equal [:id]
+    @ds.insert
+    @ds.all.must_equal [{:id=>10}, {:id=>nil}]
+  end
+
+  it "should set column defaults to nil for NOT NULL columns correctly" do
+    @db.create_table!(:items){Integer :id, :null=>false}
+    @ds.insert(:id=>10)
+    @db.alter_table(:items){set_column_default :id, nil}
+    @db.schema(:items, :reload=>true).map{|x| x.first}.must_equal [:id]
+    @ds.columns!.must_equal [:id]
+    @ds.insert(20)
+    @ds.all.must_equal [{:id=>10}, {:id=>20}]
+  end
+
+  cspecify "should set column types correctly", [:jdbc, :db2], :oracle do
     @db.create_table!(:items){Integer :id}
     @ds.insert(:id=>10)
     @db.alter_table(:items){set_column_type :id, String}
@@ -597,7 +673,7 @@ describe "Database schema modifiers" do
     @ds.order(:id).all.must_equal [{:id=>"10"}, {:id=>"20"}]
   end
 
-  cspecify "should set column types without modifying NULL/NOT NULL", [:jdbc, :db2], [:db2], :oracle, :derby do
+  cspecify "should set column types without modifying NULL/NOT NULL", [:jdbc, :db2], :derby do
     @db.create_table!(:items){Integer :id, :null=>false, :default=>2}
     proc{@ds.insert(:id=>nil)}.must_raise(Sequel::NotNullConstraintViolation, Sequel::ConstraintViolation, Sequel::DatabaseError)
     @db.alter_table(:items){set_column_type :id, String}
@@ -610,7 +686,7 @@ describe "Database schema modifiers" do
     @ds.map(:id).must_equal [nil, nil]
   end
 
-  cspecify "should set column types without modifying defaults", [:jdbc, :db2], [:db2], :oracle, :derby do
+  cspecify "should set column types without modifying defaults", [:jdbc, :db2], :oracle, :derby do
     @db.create_table!(:items){Integer :id, :default=>0}
     @ds.insert
     @ds.map(:id).must_equal [0]
@@ -683,10 +759,15 @@ describe "Database schema modifiers" do
     @db.create_table!(:items) do
       primary_key :id
       Integer :i, :default=>20
+      Integer :j, :default=>10
+      String :s, :default=>'a'
     end
-    @ds.insert(:i=>10)
+    @ds.insert(:i=>10, :j=>20, :s=>'b')
     @db.drop_column(:items, :i)
-    @db.schema(:items, :reload=>true).map{|x| x.first}.must_equal [:id]
+    @db.schema(:items, :reload=>true).map{|x| x.first}.must_equal [:id, :j, :s]
+    @ds.first.must_equal(:id=>1, :j=>20, :s=>'b')
+    @ds.insert
+    @ds.first(:id=>2).must_equal(:id=>2, :j=>10, :s=>'a')
   end
 
   it "should remove foreign key columns from tables correctly" do
@@ -715,19 +796,19 @@ describe "Database schema modifiers" do
     @db.schema(:items, :reload=>true).map{|x| x.first}.must_equal [:id]
   end
 
-  cspecify "should work correctly with many operations in a single alter_table call", [:jdbc, :db2], [:db2] do
+  cspecify "should work correctly with many operations in a single alter_table call", [:jdbc, :db2] do
     @db.create_table!(:items) do
       primary_key :id
       String :name2
       String :number2
-      constraint :bar, Sequel.~(:id=>nil)
+      constraint :bar, Sequel.~(:number2=>nil, :name2=>nil)
     end
     @ds.insert(:name2=>'A12')
     @db.alter_table(:items) do
       add_column :number, Integer
+      drop_constraint :bar
       drop_column :number2
       rename_column :name2, :name
-      drop_constraint :bar
       set_column_not_null :name
       set_column_default :name, 'A13'
       add_constraint :foo, Sequel.like(:name, 'A%')
@@ -769,18 +850,15 @@ describe "Database#tables and #views" do
       def xxxxx
         "xxxxx#{@@xxxxx += 1}"
       end
+      alias xxxxx xxxxx
     end
     @db = DB
     @db.drop_view(:sequel_test_view) rescue nil
     @db.drop_table?(:sequel_test_table)
     @db.create_table(:sequel_test_table){Integer :a}
     @db.create_view :sequel_test_view, @db[:sequel_test_table]
-    @iom = @db.identifier_output_method
-    @iim = @db.identifier_input_method
   end
   after do
-    @db.identifier_output_method = @iom
-    @db.identifier_input_method = @iim
     @db.drop_view :sequel_test_view
     @db.drop_table :sequel_test_table
   end
@@ -793,12 +871,6 @@ describe "Database#tables and #views" do
     ts.wont_include(:sequel_test_view)
   end if DB.supports_table_listing?
 
-  it "#tables should respect the database's identifier_output_method" do
-    @db.identifier_output_method = :xxxxx
-    @db.identifier_input_method = :xxxxx
-    @db.tables.each{|t| t.to_s.must_match(/\Ax{5}\d+\z/)}
-  end if DB.supports_table_listing?
-
   it "#views should return an array of symbols" do
     ts = @db.views
     ts.must_be_kind_of(Array)
@@ -807,9 +879,26 @@ describe "Database#tables and #views" do
     ts.must_include(:sequel_test_view)
   end if DB.supports_view_listing?
 
-  it "#views should respect the database's identifier_output_method" do
-    @db.identifier_output_method = :xxxxx
-    @db.identifier_input_method = :xxxxx
-    @db.views.each{|t| t.to_s.must_match(/\Ax{5}\d+\z/)}
-  end if DB.supports_view_listing?
+  describe "with identifier mangling" do
+    before do
+      @iom = @db.identifier_output_method
+      @iim = @db.identifier_input_method
+    end
+    after do
+      @db.identifier_output_method = @iom
+      @db.identifier_input_method = @iim
+    end
+
+    it "#tables should respect the database's identifier_output_method" do
+      @db.identifier_output_method = :xxxxx
+      @db.identifier_input_method = :xxxxx
+      @db.tables.each{|t| t.to_s.must_match(/\Ax{5}\d+\z/)}
+    end if DB.supports_table_listing?
+
+    it "#views should respect the database's identifier_output_method" do
+      @db.identifier_output_method = :xxxxx
+      @db.identifier_input_method = :xxxxx
+      @db.views.each{|t| t.to_s.must_match(/\Ax{5}\d+\z/)}
+    end if DB.supports_view_listing?
+  end if IDENTIFIER_MANGLING && !DB.frozen?
 end
