@@ -9,12 +9,16 @@ end
 $:.unshift(File.join(File.dirname(File.expand_path(__FILE__)), "../../lib/"))
 require_relative "../../lib/sequel"
 
+require_relative "../visibility_checking" if ENV['CHECK_METHOD_VISIBILITY']
+
 begin
   require_relative "../spec_config" unless defined?(DB)
 rescue LoadError
 end
 Sequel::Deprecation.backtrace_filter = lambda{|line, lineno| lineno < 4 || line =~ /_(spec|test)\.rb/}
 
+Sequel.extension :fiber_concurrency if ENV['SEQUEL_FIBER_CONCURRENCY']
+
 Sequel::DB = nil unless Sequel.constants.include?(:DB)
 Sequel.split_symbols = true if ENV['SEQUEL_SPLIT_SYMBOLS']
 Sequel::Database.extension :columns_introspection if ENV['SEQUEL_COLUMNS_INTROSPECTION']
@@ -46,7 +50,6 @@ DB.extension :synchronize_sql if ENV['SEQUEL_SYNCHRONIZE_SQL']
 DB.extension :integer64 if ENV['SEQUEL_INTEGER64']
 
 if ENV['SEQUEL_CONNECTION_VALIDATOR']
-  ENV['SEQUEL_NO_CHECK_SQLS'] = '1'
   DB.extension(:connection_validator)
   DB.pool.connection_validation_timeout = -1
 end
