@@ -1,4 +1,4 @@
-require File.join(File.dirname(File.expand_path(__FILE__)), "spec_helper")
+require_relative "spec_helper"
 
 describe "Sequel::Model()" do
   before do
@@ -9,7 +9,7 @@ describe "Sequel::Model()" do
     ds = @db[:blah]
     c = Sequel::Model(ds)
     c.superclass.must_equal Sequel::Model
-    c.dataset.must_equal ds
+    c.dataset.row_proc.must_equal c
   end
 
   it "should return a model subclass with a dataset with the default database and given table name if given a Symbol" do
@@ -51,14 +51,14 @@ describe "Sequel::Model()" do
     ds = @db[Sequel.identifier(:blah)]
     c = Sequel::Model(ds)
     c.superclass.must_equal Sequel::Model
-    c.dataset.must_equal ds
+    c.dataset.row_proc.must_equal c
   end
 
   it "should be callable on Sequel::Model" do
     ds = @db[:blah]
     c = Sequel::Model::Model(ds)
     c.superclass.must_equal Sequel::Model
-    c.dataset.must_equal ds
+    c.dataset.row_proc.must_equal c
   end
 
   it "should be callable on subclasses of Sequel::Model" do
@@ -66,7 +66,7 @@ describe "Sequel::Model()" do
     c = Class.new(Sequel::Model)
     sc = c::Model(ds)
     sc.superclass.must_equal c
-    sc.dataset.must_equal ds
+    sc.dataset.row_proc.must_equal sc
   end
 
   it "should be callable on other modules if def_Model is used" do
@@ -75,7 +75,7 @@ describe "Sequel::Model()" do
     ds = @db[:blah]
     c = m::Model(ds)
     c.superclass.must_equal Sequel::Model
-    c.dataset.must_equal ds
+    c.dataset.row_proc.must_equal c
   end
 
   it "should be callable using model subclasses on other modules if def_Model is used" do
@@ -85,7 +85,7 @@ describe "Sequel::Model()" do
     ds = @db[:blah]
     sc = m::Model(ds)
     sc.superclass.must_equal c
-    sc.dataset.must_equal ds
+    sc.dataset.row_proc.must_equal sc
   end
 
   it "should return a model subclass associated to the given database if given a database" do
@@ -102,10 +102,10 @@ describe "Sequel::Model()" do
 
   describe "reloading" do
     before do
-      Sequel.cache_anonymous_models = true
+      Sequel::Model.cache_anonymous_models = true
     end
     after do
-      Sequel.cache_anonymous_models = false
+      Sequel::Model.cache_anonymous_models = false
       Object.send(:remove_const, :Album) if defined?(::Album)
     end
 
@@ -150,14 +150,14 @@ describe "Sequel::Model()" do
     end
 
     it "should raise an exception if anonymous model caching is disabled" do
-      Sequel.cache_anonymous_models = false
+      Sequel::Model.cache_anonymous_models = false
       proc do
         class ::Album < Sequel::Model(@db[Sequel.identifier(:table)]); end
         class ::Album < Sequel::Model(@db[Sequel.identifier(:table)]); end
       end.must_raise TypeError
     end
 
-    it "should use separate cache and cache settings for subclasses" do
+    it "should use separate anonymous cache for subclasses" do
       c = Class.new(Sequel::Model)
       c.cache_anonymous_models.must_equal true
       class ::Album < c::Model(:table); end
@@ -174,12 +174,65 @@ describe "Sequel::Model()" do
   end
 end
 
+describe "Sequel::Model.freeze" do
+  it "should freeze the model class and not allow any changes" do
+    model = Class.new(Sequel::Model(:items))
+    model.freeze
+
+    model.frozen?.must_equal true
+    model.dataset.frozen?.must_equal true
+    model.db_schema.frozen?.must_equal true
+    model.db_schema[:id].frozen?.must_equal true
+    model.columns.frozen?.must_equal true
+    model.setter_methods.frozen?.must_equal true
+    model.send(:overridable_methods_module).frozen?.must_equal true
+    model.default_set_fields_options.frozen?.must_equal true
+
+    proc{model.dataset_module{}}.must_raise RuntimeError
+  end
+
+  it "should work if the model is already frozen" do
+    model = Class.new(Sequel::Model(:items))
+    model.freeze.freeze
+  end
+
+  it "should freeze a model class without a dataset without breaking" do
+    model = Class.new(Sequel::Model)
+    model.freeze
+    model.frozen?.must_equal true
+    proc{model.dataset}.must_raise Sequel::Error
+    model.db_schema.must_be_nil
+    model.columns.must_be_nil
+    model.setter_methods.must_equal []
+    model.send(:overridable_methods_module).frozen?.must_equal true
+    model.default_set_fields_options.frozen?.must_equal true
+
+    proc{model.dataset_module{}}.must_raise RuntimeError
+  end
+
+  it "should allow subclasses of frozen model classes to work correctly" do
+    model = Class.new(Sequel::Model(:items))
+    model.freeze
+    model = Class.new(model)
+    model.dataset = :items2
+
+    model.dataset_module{}
+    model.plugin Module.new
+
+    model.frozen?.must_equal false
+    model.db_schema.frozen?.must_equal false
+    model.db_schema[:id].frozen?.must_equal false
+    model.setter_methods.frozen?.must_equal false
+    model.dataset_module{}.frozen?.must_equal false
+    model.send(:overridable_methods_module).frozen?.must_equal false
+    model.default_set_fields_options.frozen?.must_equal false
+  end
+end
+
 describe Sequel::Model do
   it "should have class method aliased as model" do
-    Sequel::Model.instance_methods.collect{|x| x.to_s}.must_include("model")
-
     model_a = Class.new(Sequel::Model(:items))
-    model_a.new.model.wont_be_nil
+    model_a.new.model.must_be_same_as model_a
   end
 
   it "should be associated with a dataset" do
@@ -203,6 +256,11 @@ describe Sequel::Model do
     DB.reset
   end
 
+  it "should not allow dup/clone" do
+    proc{@model.dup}.must_raise NoMethodError
+    proc{@model.clone}.must_raise NoMethodError
+  end
+
   it "has table_name return name of table" do
     @model.table_name.must_equal :items
   end
@@ -216,42 +274,29 @@ describe Sequel::Model do
     @model.primary_key.must_equal :ssn
   end
 
+  it "allow not primary key change for frozen class" do
+    @model.freeze
+    proc{@model.set_primary_key :ssn}.must_raise RuntimeError
+  end
+
   it "allows dataset change" do
     @model.set_dataset(DB[:foo])
     @model.table_name.must_equal :foo
   end
 
-  it "allows set_dataset to accept a Symbol" do
-    @model.db = DB
-    @model.set_dataset(:foo)
+  it "allows frozen dataset" do
+    @model.set_dataset(DB[:foo].freeze)
     @model.table_name.must_equal :foo
+    @model.dataset.sql.must_equal 'SELECT * FROM foo'
   end
 
-  it "allows set_dataset to accept a LiteralString" do
-    @model.db = DB
-    @model.set_dataset(Sequel.lit('foo'))
-    @model.table_name.must_equal Sequel.lit('foo')
-  end
-
-  it "allows set_dataset to acceptan SQL::Identifier" do
-    @model.db = DB
-    @model.set_dataset(Sequel.identifier(:foo))
-    @model.table_name.must_equal Sequel.identifier(:foo)
-  end
-
-  it "allows set_dataset to acceptan SQL::QualifiedIdentifier" do
-    @model.db = DB
-    @model.set_dataset(Sequel.qualify(:bar, :foo))
-    @model.table_name.must_equal Sequel.qualify(:bar, :foo)
-  end
-
-  it "allows set_dataset to acceptan SQL::AliasedExpression" do
-    @model.db = DB
-    @model.set_dataset(Sequel.as(:foo, :bar))
-    @model.table_name.must_equal :bar
-  end
 
   it "table_name should respect table aliases" do
+    @model.set_dataset(Sequel[:foo].as(:x))
+    @model.table_name.must_equal :x
+  end
+  
+  with_symbol_splitting "table_name should respect table alias symbols" do
     @model.set_dataset(:foo___x)
     @model.table_name.must_equal :x
   end
@@ -260,14 +305,20 @@ describe Sequel::Model do
     proc{@model.set_dataset(Object.new)}.must_raise(Sequel::Error)
   end
 
+  it "set_dataset should use a subquery for joined datasets" do
+    @model.set_dataset(DB.from(:foo, :bar))
+    @model.dataset.sql.must_equal 'SELECT * FROM (SELECT * FROM foo, bar) AS foo'
+    @model.set_dataset(DB[:foo].cross_join(:bar))
+    @model.dataset.sql.must_equal 'SELECT * FROM (SELECT * FROM foo CROSS JOIN bar) AS foo'
+  end
+
   it "set_dataset should add the destroy method to the dataset that destroys each object" do
     ds = DB[:foo]
     ds.wont_respond_to(:destroy)
-    @model.set_dataset(ds)
+    ds = @model.set_dataset(ds).dataset
     ds.must_respond_to(:destroy)
     DB.sqls
-    ds._fetch = [{:id=>1}, {:id=>2}]
-    ds.destroy.must_equal 2
+    ds.with_fetch([{:id=>1}, {:id=>2}]).destroy.must_equal 2
     DB.sqls.must_equal ["SELECT * FROM foo", "DELETE FROM foo WHERE id = 1", "DELETE FROM foo WHERE id = 2"]
   end
 
@@ -275,7 +326,7 @@ describe Sequel::Model do
     db = Sequel.mock(:servers=>{:s1=>{}})
     ds = db[:foo].server(:s1)
     @model.use_transactions = true
-    @model.set_dataset(ds)
+    ds = @model.set_dataset(ds).dataset
     db.sqls
     ds.destroy.must_equal 0
     db.sqls.must_equal ["BEGIN -- s1", "SELECT * FROM foo -- s1", "COMMIT -- s1"]
@@ -283,12 +334,12 @@ describe Sequel::Model do
 
   it "should raise an error on set_dataset if there is an error connecting to the database" do
     def @model.columns() raise Sequel::DatabaseConnectionError end
-    proc{@model.set_dataset(Sequel::Database.new[:foo].join(:blah))}.must_raise Sequel::DatabaseConnectionError
+    proc{@model.set_dataset(Sequel::Database.new[:foo].join(:blah).from_self)}.must_raise Sequel::DatabaseConnectionError
   end
 
   it "should not raise an error if there is a problem getting the columns for a dataset" do
     def @model.columns() raise Sequel::Error end
-    @model.set_dataset(DB[:foo].join(:blah))
+    @model.set_dataset(DB[:foo].join(:blah).from_self)
   end
 
   it "doesn't raise an error on set_dataset if there is an error raised getting the schema" do
@@ -306,7 +357,7 @@ describe Sequel::Model do
   it "doesn't raise an error on inherited if there is an error setting the dataset" do
     db = Sequel.mock
     def db.schema(*) raise Sequel::Error; end
-    @model.db = db
+    @model.dataset = db[:foo]
     Class.new(@model)
   end
 
@@ -315,7 +366,7 @@ describe Sequel::Model do
     def db.supports_savepoints?; true end
     Sequel::Model(db[:table])
     db.sqls.must_equal ["SELECT * FROM table LIMIT 1"]
-    c = db.transaction{Sequel::Model(db[:table])}
+    db.transaction{Sequel::Model(db[:table])}
     db.sqls.must_equal ["BEGIN", "SAVEPOINT autopoint_1", "SELECT * FROM table LIMIT 1", "RELEASE SAVEPOINT autopoint_1", "COMMIT"]
   end
 
@@ -337,6 +388,45 @@ describe Sequel::Model do
       @d.must_equal 20
     end
   end
+
+  it "set_dataset should readd dataset method modules" do
+    m = Module.new
+    @model.dataset_module(m)
+    @model.set_dataset(@model.dataset)
+    @model.dataset.singleton_class.ancestors.must_include m
+  end
+end
+
+describe Sequel::Model do
+  before do
+    @model = Class.new(Sequel::Model)
+    DB.reset
+  end
+
+  it "allows set_dataset to accept a Symbol" do
+    @model.set_dataset(:foo)
+    @model.table_name.must_equal :foo
+  end
+
+  it "allows set_dataset to accept a LiteralString" do
+    @model.set_dataset(Sequel.lit('foo'))
+    @model.table_name.must_equal Sequel.lit('foo')
+  end
+
+  it "allows set_dataset to acceptan SQL::Identifier" do
+    @model.set_dataset(Sequel.identifier(:foo))
+    @model.table_name.must_equal Sequel.identifier(:foo)
+  end
+
+  it "allows set_dataset to acceptan SQL::QualifiedIdentifier" do
+    @model.set_dataset(Sequel.qualify(:bar, :foo))
+    @model.table_name.must_equal Sequel.qualify(:bar, :foo)
+  end
+
+  it "allows set_dataset to acceptan SQL::AliasedExpression" do
+    @model.set_dataset(Sequel.as(:foo, :bar))
+    @model.table_name.must_equal :bar
+  end
 end
 
 describe Sequel::Model, ".require_valid_table = true" do
@@ -390,6 +480,13 @@ describe Sequel::Model, ".require_valid_table = true" do
     c.set_dataset @db[:bars]
     c.columns.must_equal [:id]
   end
+
+  it "should assume nil value is the same as false" do
+    c = Class.new(Sequel::Model)
+    c.require_valid_table = nil
+    ds = @db.dataset.with_extend{def columns; raise Sequel::Error; end}
+    c.set_dataset(ds)
+  end
 end
 
 describe Sequel::Model, "constructors" do
@@ -456,8 +553,7 @@ describe Sequel::Model, "new" do
   end
 
   it "should use the last inserted id as primary key if not in values" do
-    @m.instance_dataset._fetch = @m.dataset._fetch = {:x => 1, :id => 1234}
-    @m.instance_dataset.autoid = @m.dataset.autoid = 1234
+    @m.dataset = @m.dataset.with_fetch(:x => 1, :id => 1234).with_autoid(1234)
 
     o = @m.new(:x => 1)
     o.save
@@ -469,40 +565,10 @@ describe Sequel::Model, "new" do
   end
 end
 
-describe Sequel::Model, ".subset" do
-  before do
-    @c = Class.new(Sequel::Model(:items))
-    DB.reset
-  end
-
-  it "should create a filter on the underlying dataset" do
-    proc {@c.new_only}.must_raise(NoMethodError)
-    
-    @c.subset(:new_only){age < 'new'}
-    
-    @c.new_only.sql.must_equal "SELECT * FROM items WHERE (age < 'new')"
-    @c.dataset.new_only.sql.must_equal "SELECT * FROM items WHERE (age < 'new')"
-    
-    @c.subset(:pricey){price > 100}
-    
-    @c.pricey.sql.must_equal "SELECT * FROM items WHERE (price > 100)"
-    @c.dataset.pricey.sql.must_equal "SELECT * FROM items WHERE (price > 100)"
-    
-    @c.pricey.new_only.sql.must_equal "SELECT * FROM items WHERE ((price > 100) AND (age < 'new'))"
-    @c.new_only.pricey.sql.must_equal "SELECT * FROM items WHERE ((age < 'new') AND (price > 100))"
-  end
-
-  it "should not override existing model methods" do
-    def @c.active() true end
-    @c.subset(:active, :active)
-    @c.active.must_equal true
-  end
-end
-
 describe Sequel::Model, ".find" do
   before do
     @c = Class.new(Sequel::Model(:items))
-    @c.dataset._fetch = {:name => 'sharon', :id => 1}
+    @c.dataset = @c.dataset.with_fetch(:name => 'sharon', :id => 1)
     DB.reset
   end
   
@@ -523,231 +589,6 @@ describe Sequel::Model, ".find" do
   end
 end
 
-describe Sequel::Model, ".finder" do
-  before do
-    @h = {:id=>1}
-    @db = Sequel.mock(:fetch=>@h)
-    @c = Class.new(Sequel::Model(@db[:items]))
-    @c.instance_eval do
-      def foo(a, b)
-        where(:bar=>a).order(b)
-      end
-    end
-    @o = @c.load(@h)
-    @db.sqls
-  end
-
-  it "should create a method that calls the method given and returns the first instance" do
-    @c.finder :foo
-    @c.first_foo(1, 2).must_equal @o
-    @c.first_foo(3, 4).must_equal @o
-    @db.sqls.must_equal ["SELECT * FROM items WHERE (bar = 1) ORDER BY 2 LIMIT 1", "SELECT * FROM items WHERE (bar = 3) ORDER BY 4 LIMIT 1"]
-  end
-
-  it "should work correctly when subclassing" do
-    @c.finder(:foo)
-    @sc = Class.new(@c)
-    @sc.set_dataset :foos
-    @db.sqls
-    @sc.first_foo(1, 2).must_equal @sc.load(@h)
-    @sc.first_foo(3, 4).must_equal @sc.load(@h)
-    @db.sqls.must_equal ["SELECT * FROM foos WHERE (bar = 1) ORDER BY 2 LIMIT 1", "SELECT * FROM foos WHERE (bar = 3) ORDER BY 4 LIMIT 1"]
-  end
-
-  it "should work correctly when dataset is modified" do
-    @c.finder(:foo)
-    @c.first_foo(1, 2).must_equal @o
-    @c.set_dataset :foos
-    @c.first_foo(3, 4).must_equal @o
-    @db.sqls.must_equal ["SELECT * FROM items WHERE (bar = 1) ORDER BY 2 LIMIT 1", "SELECT * FROM foos LIMIT 1", "SELECT * FROM foos WHERE (bar = 3) ORDER BY 4 LIMIT 1"]
-  end
-
-  it "should create a method based on the given block if no method symbol provided" do
-    @c.finder(:name=>:first_foo){|pl, ds| ds.where(pl.arg).limit(1)}
-    @c.first_foo(:id=>1).must_equal @o
-    @db.sqls.must_equal ["SELECT * FROM items WHERE (id = 1) LIMIT 1"]
-  end
-
-  it "should raise an error if both a block and method symbol given" do
-    proc{@c.finder(:foo, :name=>:first_foo){|pl, ds| ds.where(pl.arg)}}.must_raise(Sequel::Error)
-  end
-
-  it "should raise an error if two option hashes are provided" do
-    proc{@c.finder({:name2=>:foo}, :name=>:first_foo){|pl, ds| ds.where(pl.arg)}}.must_raise(Sequel::Error)
-  end
-
-  it "should support :type option" do
-    @c.finder :foo, :type=>:all
-    @c.finder :foo, :type=>:each
-    @c.finder :foo, :type=>:get
-
-    a = []
-    @c.all_foo(1, 2){|r| a << r}.must_equal [@o]
-    a.must_equal [@o]
-   
-    a = []
-    @c.each_foo(3, 4){|r| a << r}
-    a.must_equal [@o]
-
-    @c.get_foo(5, 6).must_equal 1
-
-    @db.sqls.must_equal ["SELECT * FROM items WHERE (bar = 1) ORDER BY 2", "SELECT * FROM items WHERE (bar = 3) ORDER BY 4", "SELECT * FROM items WHERE (bar = 5) ORDER BY 6 LIMIT 1"]
-  end
-
-  it "should support :name option" do
-    @c.finder :foo, :name=>:find_foo
-    @c.find_foo(1, 2).must_equal @o
-    @c.find_foo(3, 4).must_equal @o
-    @db.sqls.must_equal ["SELECT * FROM items WHERE (bar = 1) ORDER BY 2 LIMIT 1", "SELECT * FROM items WHERE (bar = 3) ORDER BY 4 LIMIT 1"]
-  end
-
-  it "should support :arity option" do
-    def @c.foobar(*b)
-      ds = dataset
-      b.each_with_index do |a, i|
-        ds = ds.where(i=>a)
-      end
-      ds
-    end
-    @c.finder :foobar, :arity=>1, :name=>:find_foobar_1
-    @c.finder :foobar, :arity=>2, :name=>:find_foobar_2
-    @c.find_foobar_1(:a)
-    @c.find_foobar_2(:a, :b)
-    @db.sqls.must_equal ["SELECT * FROM items WHERE (0 = a) LIMIT 1", "SELECT * FROM items WHERE ((0 = a) AND (1 = b)) LIMIT 1"]
-  end
-
-  it "should support :mod option" do
-    m = Module.new
-    @c.finder :foo, :mod=>m
-    proc{@c.first_foo}.must_raise NoMethodError
-    @c.extend m
-    @c.first_foo(1, 2).must_equal @o
-    @c.first_foo(3, 4).must_equal @o
-    @db.sqls.must_equal ["SELECT * FROM items WHERE (bar = 1) ORDER BY 2 LIMIT 1", "SELECT * FROM items WHERE (bar = 3) ORDER BY 4 LIMIT 1"]
-  end
-
-  it "should raise error when calling with the wrong arity" do
-    @c.finder :foo
-    proc{@c.first_foo(1)}.must_raise Sequel::Error
-    proc{@c.first_foo(1,2,3)}.must_raise Sequel::Error
-  end
-end
-
-describe Sequel::Model, ".prepared_finder" do
-  before do
-    @h = {:id=>1}
-    @db = Sequel.mock(:fetch=>@h)
-    @db.extend_datasets do
-      def select_sql
-        sql = super
-        sql << ' -- prepared' if is_a?(Sequel::Dataset::PreparedStatementMethods)
-        sql
-      end
-    end
-    @c = Class.new(Sequel::Model(@db[:items]))
-    @c.instance_eval do
-      def foo(a, b)
-        where(:bar=>a).order(b)
-      end
-    end
-    @o = @c.load(@h)
-    @db.sqls
-  end
-
-  it "should create a method that calls the method given and returns the first instance" do
-    @c.prepared_finder :foo
-    @c.first_foo(1, 2).must_equal @o
-    @c.first_foo(3, 4).must_equal @o
-    @db.sqls.must_equal ["SELECT * FROM items WHERE (bar = 1) ORDER BY 2 LIMIT 1 -- prepared", "SELECT * FROM items WHERE (bar = 3) ORDER BY 4 LIMIT 1 -- prepared"]
-  end
-
-  it "should work correctly when subclassing" do
-    @c.prepared_finder(:foo)
-    @sc = Class.new(@c)
-    @sc.set_dataset :foos
-    @db.sqls
-    @sc.first_foo(1, 2).must_equal @sc.load(@h)
-    @sc.first_foo(3, 4).must_equal @sc.load(@h)
-    @db.sqls.must_equal ["SELECT * FROM foos WHERE (bar = 1) ORDER BY 2 LIMIT 1 -- prepared", "SELECT * FROM foos WHERE (bar = 3) ORDER BY 4 LIMIT 1 -- prepared"]
-  end
-
-  it "should work correctly when dataset is modified" do
-    @c.prepared_finder(:foo)
-    @c.first_foo(1, 2).must_equal @o
-    @c.set_dataset :foos
-    @c.first_foo(3, 4).must_equal @o
-    @db.sqls.must_equal ["SELECT * FROM items WHERE (bar = 1) ORDER BY 2 LIMIT 1 -- prepared", "SELECT * FROM foos LIMIT 1", "SELECT * FROM foos WHERE (bar = 3) ORDER BY 4 LIMIT 1 -- prepared"]
-  end
-
-  it "should create a method based on the given block if no method symbol provided" do
-    @c.prepared_finder(:name=>:first_foo){|a1| where(:id=>a1).limit(1)}
-    @c.first_foo(1).must_equal @o
-    @db.sqls.must_equal ["SELECT * FROM items WHERE (id = 1) LIMIT 1 -- prepared"]
-  end
-
-  it "should raise an error if both a block and method symbol given" do
-    proc{@c.prepared_finder(:foo, :name=>:first_foo){|pl, ds| ds.where(pl.arg)}}.must_raise(Sequel::Error)
-  end
-
-  it "should raise an error if two option hashes are provided" do
-    proc{@c.prepared_finder({:name2=>:foo}, :name=>:first_foo){|pl, ds| ds.where(pl.arg)}}.must_raise(Sequel::Error)
-  end
-
-  it "should support :type option" do
-    @c.prepared_finder :foo, :type=>:all
-    @c.prepared_finder :foo, :type=>:each
-
-    a = []
-    @c.all_foo(1, 2){|r| a << r}.must_equal [@o]
-    a.must_equal [@o]
-   
-    a = []
-    @c.each_foo(3, 4){|r| a << r}
-    a.must_equal [@o]
-
-    @db.sqls.must_equal ["SELECT * FROM items WHERE (bar = 1) ORDER BY 2 -- prepared", "SELECT * FROM items WHERE (bar = 3) ORDER BY 4 -- prepared"]
-  end
-
-  it "should support :name option" do
-    @c.prepared_finder :foo, :name=>:find_foo
-    @c.find_foo(1, 2).must_equal @o
-    @c.find_foo(3, 4).must_equal @o
-    @db.sqls.must_equal ["SELECT * FROM items WHERE (bar = 1) ORDER BY 2 LIMIT 1 -- prepared", "SELECT * FROM items WHERE (bar = 3) ORDER BY 4 LIMIT 1 -- prepared"]
-  end
-
-  it "should support :arity option" do
-    def @c.foobar(*b)
-      ds = dataset
-      b.each_with_index do |a, i|
-        ds = ds.where(i=>a)
-      end
-      ds
-    end
-    @c.prepared_finder :foobar, :arity=>1, :name=>:find_foobar_1
-    @c.prepared_finder :foobar, :arity=>2, :name=>:find_foobar_2
-    @c.find_foobar_1(:a)
-    @c.find_foobar_2(:a, :b)
-    @db.sqls.must_equal ["SELECT * FROM items WHERE (0 = a) LIMIT 1 -- prepared", "SELECT * FROM items WHERE ((0 = a) AND (1 = b)) LIMIT 1 -- prepared"]
-  end
-
-  it "should support :mod option" do
-    m = Module.new
-    @c.prepared_finder :foo, :mod=>m
-    proc{@c.first_foo}.must_raise NoMethodError
-    @c.extend m
-    @c.first_foo(1, 2).must_equal @o
-    @c.first_foo(3, 4).must_equal @o
-    @db.sqls.must_equal ["SELECT * FROM items WHERE (bar = 1) ORDER BY 2 LIMIT 1 -- prepared", "SELECT * FROM items WHERE (bar = 3) ORDER BY 4 LIMIT 1 -- prepared"]
-  end
-
-  it "should handle models with names" do
-    def @c.name; 'foobar' end
-    @c.prepared_finder :foo
-    @c.first_foo(1, 2).must_equal @o
-    @db.sqls.must_equal ["SELECT * FROM items WHERE (bar = 1) ORDER BY 2 LIMIT 1 -- prepared"]
-  end
-end
-
 describe Sequel::Model, ".fetch" do
   before do
     DB.reset
@@ -759,9 +600,12 @@ describe Sequel::Model, ".fetch" do
   end
 
   it "should return true for .empty? and not raise an error on empty selection" do
-    rows = @c.fetch("SELECT * FROM items WHERE FALSE")
-    @c.send(:define_method, :fetch_rows){|sql| yield({:count => 0})}
-    rows.empty?
+    @c.dataset = @c.dataset.with_extend do
+      def fetch_rows(sql)
+        yield({:count => 0})
+      end
+    end
+    @c.fetch("SELECT * FROM items WHERE FALSE").empty?
   end
 end
 
@@ -788,17 +632,16 @@ describe Sequel::Model, ".find_or_create" do
     @c.find_or_create(:x => 1).must_equal @c.load(:x=>1, :id=>1)
     @db.sqls.must_equal ["SELECT * FROM items WHERE (x = 1) LIMIT 1",
       "INSERT INTO items (x) VALUES (1)",
-      "SELECT * FROM items WHERE (id = 1) LIMIT 1"]
+      "SELECT * FROM items WHERE id = 1"]
   end
 
   it "should pass the new record to be created to the block if no record is found" do
     @db.fetch = [[], {:x=>1, :id=>1}]
     @db.autoid = 1
     @c.find_or_create(:x => 1){|x| x[:y] = 2}.must_equal @c.load(:x=>1, :id=>1)
-    sqls = @db.sqls
-    sqls.first.must_equal "SELECT * FROM items WHERE (x = 1) LIMIT 1"
-    ["INSERT INTO items (x, y) VALUES (1, 2)", "INSERT INTO items (y, x) VALUES (2, 1)"].must_include(sqls[1])
-    sqls.last.must_equal "SELECT * FROM items WHERE (id = 1) LIMIT 1"
+    @db.sqls.must_equal ["SELECT * FROM items WHERE (x = 1) LIMIT 1",
+      "INSERT INTO items (x, y) VALUES (1, 2)",
+      "SELECT * FROM items WHERE id = 1"]
   end
 end
 
@@ -856,19 +699,14 @@ describe Sequel::Model, "attribute accessors" do
   end
 
   it "should be created on set_dataset" do
-    %w'x z x= z='.each do |x|
-      @c.instance_methods.collect{|z| z.to_s}.wont_include(x)
-    end
+    a = [:x, :z, :x= ,:z=]
+    (a - @c.instance_methods).must_equal a
     @c.set_dataset(@dataset)
-    %w'x z x= z='.each do |x|
-      @c.instance_methods.collect{|z| z.to_s}.must_include(x)
-    end
+    (a - @c.instance_methods).must_equal []
     o = @c.new
-    %w'x z x= z='.each do |x|
-      o.methods.collect{|z| z.to_s}.must_include(x)
-    end
+    (a - o.methods).must_equal []
 
-    o.x.must_equal nil
+    o.x.must_be_nil
     o.x = 34
     o.x.must_equal 34
   end
@@ -906,7 +744,7 @@ end
 describe Sequel::Model, ".[]" do
   before do
     @c = Class.new(Sequel::Model(:items))
-    @c.dataset._fetch = {:name => 'sharon', :id => 1}
+    @c.dataset = @c.dataset.with_fetch(:name => 'sharon', :id => 1)
     DB.reset
   end
 
@@ -918,8 +756,8 @@ describe Sequel::Model, ".[]" do
   end
 
   it "should have #[] return nil if no rows match" do
-    @c.dataset._fetch = []
-    @c[1].must_equal nil
+    @c.dataset = @c.dataset.with_fetch([])
+    @c[1].must_be_nil
     DB.sqls.must_equal ["SELECT * FROM items WHERE id = 1"]
   end
 
@@ -929,18 +767,16 @@ describe Sequel::Model, ".[]" do
     DB.sqls.must_equal ["SELECT * FROM items WHERE name = 'sharon'"]
   end
 
-  it "should use a qualified primary key if the dataset is joined" do
-    @c.dataset = @c.dataset.cross_join(:a)
+  it "should handle a dataset that uses a subquery" do
+    @c.dataset = @c.dataset.cross_join(:a).from_self(:alias=>:b)
     @c[1].must_equal @c.load(:name => 'sharon', :id => 1)
-    DB.sqls.must_equal ["SELECT * FROM items CROSS JOIN a WHERE (items.id = 1) LIMIT 1"]
+    DB.sqls.must_equal ["SELECT * FROM (SELECT * FROM items CROSS JOIN a) AS b WHERE (id = 1) LIMIT 1"]
   end
 
   it "should work correctly for composite primary key specified as array" do
     @c.set_primary_key [:node_id, :kind]
     @c[3921, 201].must_be_kind_of(@c)
-    sqls = DB.sqls
-    sqls.length.must_equal 1
-    sqls.first.must_match(/^SELECT \* FROM items WHERE \((\(node_id = 3921\) AND \(kind = 201\))|(\(kind = 201\) AND \(node_id = 3921\))\) LIMIT 1$/)
+    DB.sqls.must_equal ['SELECT * FROM items WHERE ((node_id = 3921) AND (kind = 201)) LIMIT 1']
   end
 end
 
@@ -961,16 +797,24 @@ describe "Model.db_schema" do
   end
   
   it "should not call database's schema if it isn't supported" do
+    @db.singleton_class.send(:remove_method, :supports_schema_parsing?)
     def @db.supports_schema_parsing?() false end
     def @db.schema(table, opts = {})
       raise Sequel::Error
     end
-    @dataset.instance_variable_set(:@columns, [:x, :y])
+    @dataset = @dataset.with_extend do
+      def columns
+        [:x, :y]
+      end
+    end
 
     @c.dataset = @dataset
     @c.db_schema.must_equal(:x=>{}, :y=>{})
     @c.columns.must_equal [:x, :y]
-    @c.dataset.instance_variable_get(:@columns).must_equal [:x, :y]
+
+    @c.instance_eval{@db_schema = nil}
+    @c.db_schema.must_equal(:x=>{}, :y=>{})
+    @c.columns.must_equal [:x, :y]
   end
 
   it "should use the database's schema and set the columns and dataset columns" do
@@ -980,10 +824,11 @@ describe "Model.db_schema" do
     @c.dataset = @dataset
     @c.db_schema.must_equal(:x=>{:type=>:integer}, :y=>{:type=>:string})
     @c.columns.must_equal [:x, :y]
-    @c.dataset.instance_variable_get(:@columns).must_equal [:x, :y]
+    @c.dataset.columns.must_equal [:x, :y]
   end
 
   it "should not restrict the schema for datasets with a :select option" do
+    @c.singleton_class.send(:remove_method, :columns)
     def @c.columns; [:x, :z]; end
     def @db.schema(table, opts = {})
       [[:x, {:type=>:integer}], [:y, {:type=>:string}]]
@@ -992,11 +837,39 @@ describe "Model.db_schema" do
     @c.db_schema.must_equal(:x=>{:type=>:integer}, :z=>{}, :y=>{:type=>:string})
   end
 
-  it "should fallback to fetching records if schema raises an error" do
+  it "should not raise error if setting dataset where getting schema and columns raises an error and require_valid_table is false" do
+    @c.require_valid_table = false
     def @db.schema(table, opts={})
       raise Sequel::Error
     end
-    @c.dataset = @dataset.join(:x, :id).columns(:id, :x)
+    @c.dataset = @dataset.join(:x, :id).from_self.columns(:id, :x)
+    @c.db_schema.must_equal(:x=>{}, :id=>{})
+  end
+  
+  it "should raise error if setting dataset where getting schema and columns raises an error and require_valid_table is true" do
+    @c.require_valid_table = true
+    def @db.schema(table, opts={})
+      raise Sequel::Error
+    end
+    @c.dataset = @dataset.join(:x, :id).from_self.columns(:id, :x)
+    @c.db_schema.must_equal(:x=>{}, :id=>{})
+  end
+  
+  it "should use dataset columns if getting schema raises an error and require_valid_table is false" do
+    @c.require_valid_table = false
+    def @db.schema(table, opts={})
+      raise Sequel::Error
+    end
+    @c.dataset = @dataset.join(:x, :id).from_self.columns(:id, :x)
+    @c.db_schema.must_equal(:x=>{}, :id=>{})
+  end
+  
+  it "should use dataset columns if getting schema raises an error and require_valid_table is true" do
+    @c.require_valid_table = true
+    def @db.schema(table, opts={})
+      raise Sequel::Error
+    end
+    @c.dataset = @dataset.join(:x, :id).from_self.columns(:id, :x)
     @c.db_schema.must_equal(:x=>{}, :id=>{})
   end
   
@@ -1046,7 +919,7 @@ describe "Model.db_schema" do
     @c.primary_key.must_equal :id
     @c.dataset = ds
     @c.db_schema.must_equal(:x=>{:primary_key=>false}, :y=>{:primary_key=>false})
-    @c.primary_key.must_equal nil
+    @c.primary_key.must_be_nil
   end
   
   it "should automatically set primary key for dataset selecting table.*" do
@@ -1068,6 +941,10 @@ describe "Model.db_schema" do
     @c.db_schema.must_equal(:x=>{:primary_key=>false}, :y=>{})
     @c.primary_key.must_equal :id
   end
+
+  it "should return nil if the class has no dataset" do
+    Class.new(Sequel::Model).db_schema.must_be_nil
+  end
 end
 
 describe "Model#use_transactions" do
