@@ -64,6 +64,16 @@ describe Sequel::Model, ".plugin" do
     @c.plugins.reject{|m| m != @t}.length.must_equal 1
   end
 
+  deprecated "should warn if loading the plugin with an argument" do
+    @c.plugin @t, 1
+    @c.plugins.must_include(@t)
+  end
+  
+  deprecated "should warn if loading the plugin with a block" do
+    @c.plugin(@t){}
+    @c.plugins.must_include(@t)
+  end
+
   it "should call apply and configure if the plugin responds to it, with the args and block used" do
     m = Module.new do
       def self.args; @args; end
@@ -149,6 +159,25 @@ describe Sequel::Model, ".plugin" do
     m.args.must_equal [:apply, :cm, :im, :dm, :configure, :configure]
   end
 
+  if RUBY_VERSION >= '2.7'
+    it "should handle keywords calling apply and configure" do
+      m = Module.new do
+        eval <<-END
+          def self.apply(model, name: (raise), &block)
+            model.instance_variable_set(:@apply, name)
+          end
+          def self.configure(model, name: (raise), &block)
+            model.instance_variable_set(:@configure, name)
+          end
+        END
+      end
+      
+      @c.plugin(m, name: 1)
+      @c.instance_variable_get(:@apply).must_equal 1
+      @c.instance_variable_get(:@configure).must_equal 1
+    end
+  end
+
   it "should include an InstanceMethods module in the class if the plugin includes it" do
     @c.plugin @t
     m = @c.new
@@ -247,6 +276,26 @@ describe Sequel::Plugins do
     @c.one.must_equal 1
   end
   
+  
+  if RUBY_VERSION >= '2.7'
+    it "should handle keywords when delegating" do
+      m = Module.new do
+        module self::ClassMethods
+          Sequel::Plugins.def_dataset_methods(self, :one)
+        end
+        module self::DatasetMethods
+          eval <<-END
+            def one(name: (raise))
+              name
+            end
+          END
+        end
+      end
+      @c.plugin m
+      @c.one(name: 1).must_equal 1
+    end
+  end
+
   it "should have def_dataset_methods accept an array with multiple methods" do
     m = Module.new do
       module self::ClassMethods
@@ -376,6 +425,9 @@ describe "Sequel::Plugins.def_sequel_method" do
 
     m1 = Sequel::Plugins.def_sequel_method(@m, "x", 1){|x, y| [x, y]}
     @scope.send(m1, 4).must_equal [4, nil]
+
+    m1 = Sequel::Plugins.def_sequel_method(@m, "x y", 1){2}
+    @scope.send(m1, 3).must_equal 2
   end
 
   it "should raise for unexpected expected_arity" do
@@ -390,6 +442,12 @@ describe "Sequel::Plugins.def_sequel_method" do
     proc{@scope.send(m1, 1)}.must_raise ArgumentError
   end
 
+  it "should raise if an invalid arg type is used" do
+    o = Object.new
+    def o.parameters; [[:foo]] end
+    proc{Sequel::Plugins.send(:_define_sequel_method_arg_numbers, o)}.must_raise Sequel::Error
+  end
+
   if RUBY_VERSION > '2.1'
     it "should raise for required keyword arguments for expected_arity 0 or 1" do
       proc{eval("Sequel::Plugins.def_sequel_method(@m, 'x', 0){|b:| [b, 1]}", binding)}.must_raise Sequel::Error
